@@ -20,14 +20,14 @@
  *        to the database.
  * @author Christian Grothoff
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
 #include <jansson.h>
 #include <microhttpd.h>
 #include <pthread.h>
-#include "taler_json_lib.h"
-#include "taler_mhd_lib.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_mhd_lib.h"
 #include "taler-auditor-httpd.h"
 #include "taler-auditor-httpd_deposit-confirmation.h"
 
@@ -342,7 +342,6 @@ TAH_DEPOSIT_CONFIRMATION_handler (
     if ((GNUNET_NO == res) ||
         (NULL == json))
       return MHD_YES;
-    dc.master_sig = es.master_sig;
     res = TALER_MHD_parse_json_data (connection,
                                      json,
                                      spec);
@@ -356,6 +355,7 @@ TAH_DEPOSIT_CONFIRMATION_handler (
       json_decref (json);
       return MHD_YES;       /* failure */
     }
+    dc.master_sig = es.master_sig;
   }
   num_coins = json_array_size (jcoin_sigs);
   if (num_coins != json_array_size (jcoin_pubs))
