@@ -18,10 +18,10 @@
  * @brief Low-level (statement-level) Postgres database access for the exchange
  * @author Christian Grothoff
  */
-#include "platform.h"
-#include "taler_error_codes.h"
-#include "taler_dbevents.h"
-#include "taler_pq_lib.h"
+#include "taler/platform.h"
+#include "taler/taler_error_codes.h"
+#include "taler/taler_dbevents.h"
+#include "taler/taler_pq_lib.h"
 #include "pg_get_denomination_balance.h"
 #include "pg_helper.h"
 
