@@ -17,8 +17,8 @@
 #ifndef SRC_PG_GET_RESERVE_BALANCE_INSUFFICIENT_INCONSISTENCY_H
 #define SRC_PG_GET_RESERVE_BALANCE_INSUFFICIENT_INCONSISTENCY_H
 
-#include "taler_util.h"
-#include "taler_auditordb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_auditordb_plugin.h"
 
 /**
  * Get information about reserve balance insufficient inconsistency from the database.
