@@ -18,9 +18,9 @@
 #ifndef SRC_PG_GET_WIRE_OUT_INCONSISTENCY_H
 #define SRC_PG_GET_WIRE_OUT_INCONSISTENCY_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_auditordb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_auditordb_plugin.h"
 
 /**
  * Get information about wire-out-inconsistency from the database.
