@@ -21,9 +21,9 @@
 #ifndef PG_INSERT_PURSE_INFO_H
 #define PG_INSERT_PURSE_INFO_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_auditordb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_auditordb_plugin.h"
 
 /**
  * Insert information about a purse.  There must not be an
