@@ -16,9 +16,9 @@
 #ifndef PG_LOOKUP_RESERVE_IN_INCONSISTENCY_H
 #define PG_LOOKUP_RESERVE_IN_INCONSISTENCY_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_auditordb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_auditordb_plugin.h"
 
 /**
  * Lookup information about reserve-in-inconsistency from the database.
