@@ -206,17 +206,9 @@ TALER_BANK_account_token (
   ath = GNUNET_new (struct TALER_BANK_AccountTokenHandle);
   ath->cb = res_cb;
   ath->cb_cls = res_cb_cls;
-  {
-    char *path;
-
-    GNUNET_asprintf (&path,
-                     "accounts/%s/token",
-                     account_name);
-    ath->request_url = TALER_url_join (auth->wire_gateway_url,
-                                       path,
-                                       NULL);
-    GNUNET_free (path);
-  }
+  ath->request_url = TALER_url_join (auth->core_bank_url,
+                                     "token",
+                                     NULL);
   if (NULL == ath->request_url)
   {
     GNUNET_free (ath);
@@ -226,10 +218,6 @@ TALER_BANK_account_token (
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Requesting access token at `%s'\n",
               ath->request_url);
-  ath->post_ctx.headers
-    = curl_slist_append (
-        ath->post_ctx.headers,
-        "Content-Type: application/json");
   eh = curl_easy_init ();
   if ( (NULL == eh) ||
        (GNUNET_OK !=
