@@ -23,10 +23,10 @@
  * @author Christian Grothoff
  * @author Marcello Stanisci
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include "bank_api_common.h"
 #include <microhttpd.h> /* just for HTTP status codes */
-#include "taler_signatures.h"
+#include "taler/taler_signatures.h"
 
 
 /**
@@ -252,13 +252,13 @@ TALER_BANK_debit_history (struct GNUNET_CURL_Context *ctx,
          (! GNUNET_TIME_relative_is_zero (timeout)) )
       GNUNET_snprintf (url,
                        sizeof (url),
-                       "history/outgoing?delta=%lld&long_poll_ms=%llu",
+                       "history/outgoing?limit=%lld&long_poll_ms=%llu",
                        (long long) num_results,
                        tms);
     else
       GNUNET_snprintf (url,
                        sizeof (url),
-                       "history/outgoing?delta=%lld",
+                       "history/outgoing?limit=%lld",
                        (long long) num_results);
   }
   else
@@ -267,14 +267,14 @@ TALER_BANK_debit_history (struct GNUNET_CURL_Context *ctx,
          (! GNUNET_TIME_relative_is_zero (timeout)) )
       GNUNET_snprintf (url,
                        sizeof (url),
-                       "history/outgoing?delta=%lld&start=%llu&long_poll_ms=%llu",
+                       "history/outgoing?limit=%lld&offset=%llu&long_poll_ms=%llu",
                        (long long) num_results,
                        (unsigned long long) start_row,
                        tms);
     else
       GNUNET_snprintf (url,
                        sizeof (url),
-                       "history/outgoing?delta=%lld&start=%llu",
+                       "history/outgoing?limit=%lld&offset=%llu",
                        (long long) num_results,
                        (unsigned long long) start_row);
   }
