@@ -40,6 +40,22 @@ TALER_BANK_auth_parse_cfg (const struct GNUNET_CONFIGURATION_Handle *cfg,
   };
   char *method;
 
+  auth->core_bank_url = NULL;
+  if (GNUNET_OK ==
+      GNUNET_CONFIGURATION_get_value_string (cfg,
+                                             section,
+                                             "CORE_BANK_URL",
+                                             &auth->core_bank_url))
+  {
+    if (! TALER_is_web_url (auth->core_bank_url))
+    {
+      GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
+                                 section,
+                                 "CORE_BANK_URL",
+                                 "Not a valid URL");
+      return GNUNET_SYSERR;
+    }
+  }
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (cfg,
                                              section,
@@ -51,7 +67,6 @@ TALER_BANK_auth_parse_cfg (const struct GNUNET_CONFIGURATION_Handle *cfg,
                                "WIRE_GATEWAY_URL");
     return GNUNET_SYSERR;
   }
-
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (cfg,
                                              section,
