@@ -22,15 +22,15 @@
  * @author Christian Grothoff <christian@grothoff.org>
  * @defgroup request Request handling routines
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <pthread.h>
 #include <poll.h>
 #ifdef __linux__
 #include <sys/eventfd.h>
 #endif
-#include "taler_fakebank_lib.h"
-#include "taler_bank_service.h"
-#include "taler_mhd_lib.h"
+#include "taler/taler_fakebank_lib.h"
+#include "taler/taler_bank_service.h"
+#include "taler/taler_mhd_lib.h"
 #include <gnunet/gnunet_mhd_compat.h>
 #include "fakebank.h"
 #include "fakebank_bank.h"
