@@ -21,10 +21,10 @@
  * @brief handle a GET "/" request for the bank API
  * @author Christian Grothoff <christian@grothoff.org>
  */
-#include "platform.h"
-#include "taler_fakebank_lib.h"
-#include "taler_bank_service.h"
-#include "taler_mhd_lib.h"
+#include "taler/platform.h"
+#include "taler/taler_fakebank_lib.h"
+#include "taler/taler_bank_service.h"
+#include "taler/taler_mhd_lib.h"
 #include <gnunet/gnunet_mhd_compat.h>
 #include "fakebank.h"
 #include "fakebank_bank_get_root.h"
