@@ -21,11 +21,11 @@
  * @brief Implementation of the GET /withdrawal-operation/ request of the Taler Bank Integration API
  * @author Christian Grothoff <christian@grothoff.org>
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <pthread.h>
-#include "taler_fakebank_lib.h"
-#include "taler_bank_service.h"
-#include "taler_mhd_lib.h"
+#include "taler/taler_fakebank_lib.h"
+#include "taler/taler_bank_service.h"
+#include "taler/taler_mhd_lib.h"
 #include <gnunet/gnunet_mhd_compat.h>
 #include "fakebank.h"
 #include "fakebank_common_lookup.h"
