@@ -92,6 +92,11 @@ static struct TALER_BANK_CreditHistoryHandle *chh;
  */
 static struct TALER_BANK_DebitHistoryHandle *dhh;
 
+/**
+ * Handle to fetch an access token.
+ */
+static struct TALER_BANK_AccountTokenHandle *ath;
+
 /**
  * Handle for executing the wire transfer.
  */
@@ -133,6 +138,11 @@ do_shutdown (void *cls)
     TALER_BANK_debit_history_cancel (dhh);
     dhh = NULL;
   }
+  if (NULL != ath)
+  {
+    TALER_BANK_account_token_cancel (ath);
+    ath = NULL;
+  }
   if (NULL != eh)
   {
     TALER_BANK_transfer_cancel (eh);
@@ -553,6 +563,76 @@ execute_admin_transfer (void)
 }
 
 
+/**
+ * Run the actual main operation requested by the user.
+ */
+static void
+execute_tasks (void)
+{
+  if (GNUNET_YES == incoming_history)
+  {
+    execute_credit_history ();
+    return;
+  }
+  if (GNUNET_YES == outgoing_history)
+  {
+    execute_debit_history ();
+    return;
+  }
+  if (NULL != credit_account.full_payto)
+  {
+    execute_wire_transfer ();
+    return;
+  }
+  if (NULL != debit_account.full_payto)
+  {
+    execute_admin_transfer ();
+    return;
+  }
+
+  GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+              "No operation specified.\n");
+  global_ret = 0;
+  GNUNET_SCHEDULER_shutdown ();
+}
+
+
+/**
+ * Receives an access token to the bank.
+ *
+ * @param cls closure
+ * @param atr response details
+ */
+static void
+access_token_cb (
+  void *cls,
+  const struct TALER_BANK_AccountTokenResponse *atr)
+{
+  (void) cls;
+  ath = NULL;
+  switch (atr->ec)
+  {
+  case TALER_EC_NONE:
+    break; /* continued below */
+  default:
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to get access token: %s (%u/%d)\n",
+                TALER_ErrorCode_get_hint (atr->ec),
+                atr->http_status,
+                (int) atr->ec);
+    global_ret = EXIT_NOPERMISSION;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+  GNUNET_assert (TALER_BANK_AUTH_BASIC == auth.method);
+  GNUNET_free (auth.details.basic.username);
+  GNUNET_free (auth.details.basic.password);
+  auth.method = TALER_BANK_AUTH_BEARER;
+  auth.details.bearer.token = GNUNET_strdup (atr->details.ok.access_token);
+  execute_tasks ();
+}
+
+
 /**
  * Main function that will be run.
  *
@@ -567,6 +647,7 @@ run (void *cls,
      const char *cfgfile,
      const struct GNUNET_CONFIGURATION_Handle *cfg)
 {
+  enum TALER_BANK_TokenScope scope;
   (void) cls;
   (void) args;
   (void) cfgfile;
@@ -653,31 +734,38 @@ run (void *cls,
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  if (GNUNET_YES == incoming_history)
+  if ( (NULL != auth.core_bank_url) &&
+       (TALER_BANK_AUTH_BASIC == auth.method) )
   {
-    execute_credit_history ();
-    return;
+    scope = TALER_BANK_TOKEN_SCOPE_READONLY;
+    if (NULL != credit_account.full_payto)
+      scope = TALER_BANK_TOKEN_SCOPE_WIREGATEWAY;
+    if (NULL != debit_account.full_payto)
+      scope = TALER_BANK_TOKEN_SCOPE_READWRITE;
+    ath = TALER_BANK_account_token (ctx,
+                                    &auth,
+                                    auth.details.basic.username, // FIXME: why? correct?
+                                    scope,
+                                    false, /* refreshable */
+                                    "taler-exchange-wire-gateway-client CLI token",
+                                    GNUNET_TIME_UNIT_MINUTES,
+                                    &access_token_cb,
+                                    NULL);
+    if (NULL == ath)
+    {
+      GNUNET_break (0);
+      GNUNET_SCHEDULER_shutdown ();
+      return;
+    }
   }
-  if (GNUNET_YES == outgoing_history)
+  else
   {
-    execute_debit_history ();
-    return;
+    if (TALER_BANK_AUTH_BASIC == auth.method)
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "No CORE_BANK_URL given in `%s' and using basic authentication. Not all taler-wire-gateway implementations allow this.\n",
+                  account_section);
+    execute_tasks ();
   }
-  if (NULL != credit_account.full_payto)
-  {
-    execute_wire_transfer ();
-    return;
-  }
-  if (NULL != debit_account.full_payto)
-  {
-    execute_admin_transfer ();
-    return;
-  }
-
-  GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-              "No operation specified.\n");
-  global_ret = 0;
-  GNUNET_SCHEDULER_shutdown ();
 }
 
 
