@@ -18,11 +18,11 @@
  * @brief Support for operations involving the auditor's (offline) key.
  * @author Christian Grothoff
  */
-#include <platform.h>
+#include "taler/platform.h"
 #include <gnunet/gnunet_json_lib.h>
 #include <microhttpd.h>
-#include "taler_json_lib.h"
-#include "taler_exchange_service.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchange_service.h"
 
 /**
  * Name of the input of a denomination key signature for the 'upload' operation.
