@@ -57,7 +57,7 @@
 #include "taler-exchange-httpd_kyc-webhook.h"
 #include "taler-exchange-httpd_aml-decision.h"
 #include "taler-exchange-httpd_management.h"
-#include "taler-exchange-httpd_melt_v27.h"
+#include "taler-exchange-httpd_melt.h"
 #include "taler-exchange-httpd_metrics.h"
 #include "taler-exchange-httpd_mhd.h"
 #include "taler-exchange-httpd_purses_create.h"
@@ -1745,7 +1745,7 @@ handle_mhd_request (void *cls,
     {
       .url = "melt",
       .method = MHD_HTTP_METHOD_POST,
-      .handler.post = &TEH_handler_melt_v27,
+      .handler.post = &TEH_handler_melt,
       .nargs = 0
     },
     /* tracking transfers */
@@ -2752,7 +2752,7 @@ do_shutdown (void *cls)
   TEH_resume_keys_requests (true);
   TEH_batch_deposit_cleanup ();
   TEH_withdraw_cleanup ();
-  TEH_melt_v27_cleanup ();
+  TEH_melt_cleanup ();
   TEH_reserves_close_cleanup ();
   TEH_reserves_purse_cleanup ();
   TEH_purses_merge_cleanup ();
