@@ -658,7 +658,7 @@ TEH_handler_aml_attributes_get (
                      json_object_set_new (rctx->details.pdf.global_attrs,
                                           "FILE_NUMBER",
                                           json_integer (file_number)));
-      /* Kookup latest AML decision & account rules & properties
+      /* Lookup latest AML decision & account rules & properties
          to build the cover page */
       qs = TEH_plugin->lookup_aml_history (TEH_plugin->cls,
                                            &h_payto,
@@ -734,12 +734,14 @@ TEH_handler_aml_attributes_get (
         TALER_EC_GENERIC_DB_FETCH_FAILED,
         "select_aml_attributes");
     case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
-      return TALER_MHD_reply_static (
-        rc->connection,
-        MHD_HTTP_NO_CONTENT,
-        NULL,
-        NULL,
-        0);
+      if (RCF_JSON == rctx->format)
+        return TALER_MHD_reply_static (
+          rc->connection,
+          MHD_HTTP_NO_CONTENT,
+          NULL,
+          NULL,
+          0);
+      break;
     case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
       break;
     }
@@ -774,10 +776,6 @@ TEH_handler_aml_attributes_get (
                        );
       }
     }
-    GNUNET_CONTAINER_DLL_insert (rctx_head,
-                                 rctx_tail,
-                                 rctx);
-    MHD_suspend_connection (rc->connection);
     rctx->tc = TALER_MHD_typst (TEH_cfg,
                                 false,
                                 "exchange",
@@ -795,6 +793,10 @@ TEH_handler_aml_attributes_get (
         TALER_EC_EXCHANGE_GENERIC_NO_TYPST_OR_PDFTK,
         NULL);
     }
+    GNUNET_CONTAINER_DLL_insert (rctx_head,
+                                 rctx_tail,
+                                 rctx);
+    MHD_suspend_connection (rc->connection);
     return MHD_YES;
   }
   GNUNET_assert (0);
