@@ -273,6 +273,18 @@ TEH_handler_post_aml_decision (
       ret = MHD_YES /* failure */;
       goto done;
     }
+    if ( (NULL != attributes) &&
+         (! json_is_string (json_object_get (attributes,
+                                             "FORM_ID"))) )
+    {
+      GNUNET_break_op (0);
+      ret = TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_BAD_REQUEST,
+        TALER_EC_GENERIC_PARAMETER_MALFORMED,
+        "attributes['FORM_ID']");
+      goto done;
+    }
   }
   if (NULL != payto_uri.full_payto)
   {
