@@ -286,7 +286,7 @@ compile_transaction_history (
         }
       }
       break;
-    case TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP:
+    case TALER_EXCHANGEDB_TT_RECOUP_REFRESH_RECEIVER:
       {
         struct TALER_EXCHANGEDB_RecoupRefreshListEntry *pr =
           pos->details.old_coin_recoup;
@@ -317,7 +317,7 @@ compile_transaction_history (
               history,
               GNUNET_JSON_PACK (
                 GNUNET_JSON_pack_string ("type",
-                                         "OLD-COIN-RECOUP"),
+                                         "RECOUP-REFRESH-RECEIVER"),
                 GNUNET_JSON_pack_uint64 ("history_offset",
                                          pos->coin_history_id),
                 TALER_JSON_pack_amount ("amount",
@@ -337,7 +337,7 @@ compile_transaction_history (
         }
         break;
       }
-    case TALER_EXCHANGEDB_TT_RECOUP:
+    case TALER_EXCHANGEDB_TT_RECOUP_WITHDRAW:
       {
         const struct TALER_EXCHANGEDB_RecoupListEntry *recoup =
           pos->details.recoup;
@@ -363,7 +363,7 @@ compile_transaction_history (
               history,
               GNUNET_JSON_PACK (
                 GNUNET_JSON_pack_string ("type",
-                                         "RECOUP"),
+                                         "RECOUP-WITHDRAW"),
                 GNUNET_JSON_pack_uint64 ("history_offset",
                                          pos->coin_history_id),
                 TALER_JSON_pack_amount ("amount",
