@@ -18,16 +18,16 @@
  * @brief Handle wire deposit tracking-related requests
  * @author Christian Grothoff
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
 #include <microhttpd.h>
 #include <pthread.h>
-#include "taler_dbevents.h"
-#include "taler_json_lib.h"
-#include "taler_util.h"
-#include "taler_mhd_lib.h"
-#include "taler_signatures.h"
+#include "taler/taler_dbevents.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_util.h"
+#include "taler/taler_mhd_lib.h"
+#include "taler/taler_signatures.h"
 #include "taler-exchange-httpd_keys.h"
 #include "taler-exchange-httpd_deposits_get.h"
 #include "taler-exchange-httpd_responses.h"
