@@ -17,16 +17,16 @@
  * @brief Handle extensions (age-restriction, policy extensions)
  * @author Özgür Kesim
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <gnunet/gnunet_json_lib.h>
-#include "taler_dbevents.h"
+#include "taler/taler_dbevents.h"
 #include "taler-exchange-httpd_keys.h"
 #include "taler-exchange-httpd_responses.h"
 #include "taler-exchange-httpd_extensions.h"
-#include "taler_extensions_policy.h"
-#include "taler_json_lib.h"
-#include "taler_mhd_lib.h"
-#include "taler_extensions.h"
+#include "taler/taler_extensions_policy.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_mhd_lib.h"
+#include "taler/taler_extensions.h"
 #include <jansson.h>
 
 /**
