@@ -130,9 +130,11 @@ struct KycPoller
   unsigned int response_code;
 
   /**
-   * True if our @e h_payto is for a wallet.
+   * #GNUNET_YES if our @e h_payto is for a wallet,
+   * #GNUNET_NO if it is for an account,
+   * #GNUNET_SYSERR if we do not know.
    */
-  bool is_wallet;
+  enum GNUNET_GenericReturnValue is_wallet;
 
   /**
    * True if we are still suspended.
@@ -578,7 +580,11 @@ current_rules_cb (
     }
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Making applicable zero-measures for %s under current rules active\n",
-                kyp->is_wallet ? "wallets" : "accounts");
+                (GNUNET_SYSERR == kyp->is_wallet)
+                ? "unknown account type"
+                : ( (GNUNET_YES == kyp->is_wallet)
+                    ? "wallets"
+                    : "accounts"));
     json_dumpf (jmeasures,
                 stderr,
                 JSON_INDENT (2));
@@ -691,6 +697,8 @@ TEH_handler_kyc_info (
 
   if (NULL == kyp)
   {
+    bool bis_wallet;
+
     kyp = GNUNET_new (struct KycPoller);
     kyp->connection = rc->connection;
     rc->rh_ctx = kyp;
@@ -747,11 +755,12 @@ TEH_handler_kyc_info (
     } /* etag */
 
     /* Check access token */
+    kyp->is_wallet = GNUNET_SYSERR;
     qs = TEH_plugin->lookup_h_payto_by_access_token (
       TEH_plugin->cls,
       &kyp->access_token,
       &kyp->h_payto,
-      &kyp->is_wallet);
+      &bis_wallet);
     if (qs < 0)
     {
       GNUNET_break (0);
@@ -769,6 +778,7 @@ TEH_handler_kyc_info (
         TALER_JSON_pack_ec (
           TALER_EC_EXCHANGE_KYC_INFO_AUTHORIZATION_FAILED));
     }
+    kyp->is_wallet = (bis_wallet) ? GNUNET_YES : GNUNET_NO;
 
     if (GNUNET_TIME_absolute_is_future (kyp->timeout))
     {
