@@ -18,16 +18,16 @@
  * @brief Handle request for proof for KYC check.
  * @author Christian Grothoff
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
 #include <jansson.h>
 #include <microhttpd.h>
-#include "taler_attributes.h"
-#include "taler_json_lib.h"
-#include "taler_kyclogic_lib.h"
-#include "taler_mhd_lib.h"
-#include "taler_templating_lib.h"
+#include "taler/taler_attributes.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_kyclogic_lib.h"
+#include "taler/taler_mhd_lib.h"
+#include "taler/taler_templating_lib.h"
 #include "taler-exchange-httpd_common_kyc.h"
 #include "taler-exchange-httpd_kyc-proof.h"
 #include "taler-exchange-httpd_responses.h"
