@@ -23,12 +23,12 @@
  * @author Benedikt Mueller
  * @author Christian Grothoff
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
 #include <microhttpd.h>
 #include <pthread.h>
-#include "taler_mhd_lib.h"
+#include "taler/taler_mhd_lib.h"
 #include "taler-exchange-httpd_responses.h"
 #include "taler-exchange-httpd.h"
 #include "taler-exchange-httpd_mhd.h"
