@@ -51,15 +51,35 @@ struct MeltRevealContext
    * The data from the original melt.  Will be retrieved from
    * the DB via @a rc.
    */
-  struct TALER_EXCHANGEDB_Refresh_v27 refresh;
+  struct TALER_EXCHANGEDB_Refresh_vDOLDPLUS refresh;
 
   /**
+   * True, if @e signatures were not provided in the request.
+   */
+  bool no_signatures;
+
+  /**
+   * @since v27
+   * @deprecated after vDOLDPLUS
+   *
    * TALER_CNC_KAPPA-1 disclosed signatures for public refresh nonces.
    */
   struct TALER_PrivateRefreshNonceSignatureP signatures[KAPPA_MINUS_1];
 
   /**
-   * False, if no age commitment was provided
+   * True, if @e transfer_secret_seeds were not provided in the request.
+   */
+  bool no_transfer_secret_seeds;
+
+  /**
+   * @since vDOLDPLUS
+   *
+   * The transfer secret seeds for the revealed batches of coins.
+   */
+  struct TALER_PrivateRefreshBatchSeedP transfer_secret_seeds[KAPPA_MINUS_1];
+
+  /**
+   * True, if no @e age_commitment was provided
    */
   bool no_age_commitment;
 
@@ -88,7 +108,7 @@ static enum GNUNET_GenericReturnValue
 find_original_refresh (
   struct MHD_Connection *connection,
   const struct TALER_RefreshCommitmentP *rc,
-  struct TALER_EXCHANGEDB_Refresh_v27 *refresh,
+  struct TALER_EXCHANGEDB_Refresh_vDOLDPLUS *refresh,
   MHD_RESULT *result)
 {
   enum GNUNET_DB_QueryStatus qs;
@@ -280,7 +300,7 @@ calculate_blinded_detail (
 /**
  * @brief Checks the validity of the disclosed signatures as follows:
  * - Verifies the validity of the disclosed signatures with the old coin's public key
- * - Derives the seeds from those signatures for disclosed fresh coins
+ * - Derives the seeds for disclosed fresh coins
  * - Derives the fresh coins from the seeds
  * - Derives new age commitment
  * - Calculates the blinded coin planchet hashes
@@ -296,16 +316,18 @@ calculate_blinded_detail (
  * @param con HTTP-connection to the client
  * @param rf Original refresh object from the previous /melt request
  * @param old_age_commitment The age commitment of the original coin
- * @param signatures The secrets of the disclosed coins, KAPPA_MINUS_1*num_coins many
+ * @param signatures The secrets of the disclosed coins, KAPPA_MINUS_1*num_coins many, maybe NULL
+ * @param rev_batch_seeds The seeds for the transfer secrets of the disclosed coins, KAPPA_MINUS_1 many, maybe NULL
  * @param[out] result On error, a HTTP-response will be queued and result set accordingly
  * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
  */
 static enum GNUNET_GenericReturnValue
 verify_commitment (
   struct MHD_Connection *con,
-  const struct TALER_EXCHANGEDB_Refresh_v27 *rf,
+  const struct TALER_EXCHANGEDB_Refresh_vDOLDPLUS *rf,
   const struct TALER_AgeCommitment *old_age_commitment,
-  const struct TALER_PrivateRefreshNonceSignatureP signatures[KAPPA_MINUS_1],
+  const struct TALER_PrivateRefreshNonceSignatureP (*signatures)[KAPPA_MINUS_1],
+  const struct TALER_PrivateRefreshBatchSeedP (*rev_batch_seeds)[KAPPA_MINUS_1],
   MHD_RESULT *result)
 {
   enum GNUNET_GenericReturnValue ret;
@@ -314,7 +336,6 @@ verify_commitment (
   struct TALER_DenominationHashP  *denoms_h[rf->num_coins];
   struct TALER_Amount total_amount;
   struct TALER_Amount total_fee;
-  struct TALER_KappaPublicRefreshNoncesP kappa_nonces;
   bool is_cs[rf->num_coins];
   size_t cs_count = 0;
 
@@ -332,6 +353,33 @@ verify_commitment (
           0,
           sizeof(is_cs));
 
+  /**
+   * Consistency check:
+   * If the refresh was for v27, signatures must not be NULL,
+   * otherwise the revealed batch seeds must not be NULL.
+   */
+  if (rf->is_v27_refresh)
+  {
+    if (NULL == signatures)
+    {
+      *result = TALER_MHD_reply_with_error (con,
+                                            MHD_HTTP_BAD_REQUEST,
+                                            TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                            "signatures missing");
+      return GNUNET_SYSERR;
+    }
+  }
+  else     /* vDOLDPLUS */
+  {
+    if (NULL == rev_batch_seeds)
+    {
+      *result = TALER_MHD_reply_with_error (con,
+                                            MHD_HTTP_BAD_REQUEST,
+                                            TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                            "batch_seeds missing");
+      return GNUNET_SYSERR;
+    }
+  }
   /**
    * We need the current keys in memory for the meta-data of the denominations
    */
@@ -394,23 +442,26 @@ verify_commitment (
                                        NULL);
     return GNUNET_SYSERR;
   }
-
   /**
-   * We expand the provided refresh_seed from the original call to /melt,
-   * into kappa many batch seeds, from which we will later use all except the
-   * noreveal_index one.
-   */
-  TALER_refresh_expand_kappa_nonces (
-    &rf->refresh_seed,
-    &kappa_nonces);
-
-  /**
-   * First things first: Verify the signature of the old coin
+   * First things first for v27 clients: Verify the signature of the old coin
    * over the refresh nonce.  This proves the ownership
    * for the fresh coin.
    */
+  if (rf->is_v27_refresh)
   {
     size_t sig_idx = 0;
+    struct TALER_KappaPublicRefreshNoncesP kappa_nonces;
+
+    GNUNET_assert (NULL != signatures);
+
+    /**
+     * We expand the provided refresh_seed from the original call to /melt,
+     * into kappa many batch seeds, from which we will later use all except the
+     * noreveal_index one.
+     */
+    TALER_refresh_expand_kappa_nonces_v27 (
+      &rf->refresh_seed,
+      &kappa_nonces);
 
     for (uint8_t k=0; k < TALER_CNC_KAPPA; k++)
     {
@@ -423,7 +474,7 @@ verify_commitment (
             rf->num_coins,
             denoms_h,
             k,
-            &signatures[sig_idx++]))
+            &(*signatures)[sig_idx++]))
       {
         GNUNET_break_op (0);
         *result = TALER_MHD_reply_with_ec (con,
@@ -433,7 +484,6 @@ verify_commitment (
       }
     }
   }
-
   /**
    * In the following scope, we start collecting blinded coin planchet hashes,
    * either those persisted from the original request to /melt, or we
@@ -449,7 +499,11 @@ verify_commitment (
     union GNUNET_CRYPTO_BlindSessionNonce b_nonces[GNUNET_NZL (cs_count)];
     size_t cs_idx = 0; /* [0...cs_count) */
     uint8_t sig_idx = 0; /* [0..KAPPA_MINUS_1) */
-
+    /* These two are only necessary for non-v27 clients, but this is the common case. */
+    struct TALER_TransferPublicKeyP k_tpbs[TALER_CNC_KAPPA][rf->num_coins];
+    struct TALER_KappaTransferPublicKeys kappa_transfer_pubs = {
+      .num_transfer_pubs = rf->num_coins
+    };
     /**
      * First, derive the blinding nonces for the CS denominations all at once.
      */
@@ -487,24 +541,40 @@ verify_commitment (
          * the disclosed input material and generate the
          * hashes of them.
          */
-        struct TALER_PlanchetMasterSecretP secrets[rf->num_coins];
+        struct TALER_PlanchetMasterSecretP planchet_secrets[rf->num_coins];
         struct TALER_PlanchetDetail details[rf->num_coins];
 
-        memset (secrets,
+        memset (planchet_secrets,
                 0,
-                sizeof(secrets));
+                sizeof(planchet_secrets));
         memset (details,
                 0,
                 sizeof(details));
+
+        GNUNET_assert (sig_idx < KAPPA_MINUS_1);
+
         /**
          * Expand from the k-th signature all num_coin planchet secrets,
          * except for the noreveal_index.
          */
-        GNUNET_assert (sig_idx < KAPPA_MINUS_1);
-        TALER_refresh_signature_to_secrets (
-          &signatures[sig_idx++],
-          rf->num_coins,
-          secrets);
+        if (rf->is_v27_refresh)
+        {
+          TALER_refresh_signature_to_secrets_v27 (
+            &(*signatures)[sig_idx++],
+            rf->num_coins,
+            planchet_secrets);
+        }
+        else        /* vDOLDPLUS */
+        {
+          TALER_refresh_expand_batch_seed_to_transfer_data (
+            &(*rev_batch_seeds)[sig_idx++],
+            &rf->coin.coin_pub,
+            rf->num_coins,
+            planchet_secrets,
+            k_tpbs[k]);
+
+          kappa_transfer_pubs.batch[k] = k_tpbs[k];
+        }
         /**
          * Reset the index for the CS  denominations.
          */
@@ -529,7 +599,7 @@ verify_commitment (
           }
           ret = calculate_blinded_detail (con,
                                           denom_keys[coin_idx],
-                                          &secrets[coin_idx],
+                                          &planchet_secrets[coin_idx],
                                           rp,
                                           np,
                                           old_age_commitment,
@@ -556,14 +626,30 @@ verify_commitment (
     {
       struct TALER_RefreshCommitmentP rc;
 
-      TALER_refresh_get_commitment_v27 (&rc,
-                                        &rf->refresh_seed,
-                                        rf->no_blinding_seed
+      if (rf->is_v27_refresh)
+      {
+        TALER_refresh_get_commitment_v27 (&rc,
+                                          &rf->refresh_seed,
+                                          rf->no_blinding_seed
                                             ? NULL
                                             : &rf->blinding_seed,
-                                        &kappa_planchets_h,
-                                        &rf->coin.coin_pub,
-                                        &rf->amount_with_fee);
+                                          &kappa_planchets_h,
+                                          &rf->coin.coin_pub,
+                                          &rf->amount_with_fee);
+      }
+      else
+      {
+        TALER_refresh_get_commitment (&rc,
+                                      &rf->refresh_seed,
+                                      rf->no_blinding_seed
+                                            ? NULL
+                                            : &rf->blinding_seed,
+                                      &kappa_transfer_pubs,
+                                      &kappa_planchets_h,
+                                      &rf->coin.coin_pub,
+                                      &rf->amount_with_fee);
+      }
+
       if (0 != GNUNET_CRYPTO_hash_cmp (
             &rf->rc.session_hash,
             &rc.session_hash))
@@ -571,7 +657,7 @@ verify_commitment (
         GNUNET_break_op (0);
         *result = TALER_MHD_reply_with_ec (con,
                                            TALER_EC_EXCHANGE_REFRESHES_REVEAL_INVALID_RCH,
-                                           NULL);
+                                           "rc");
         return GNUNET_SYSERR;
       }
     }
@@ -580,6 +666,61 @@ verify_commitment (
 }
 
 
+/**
+ * @brief Commit the successful reveal to the database
+ *
+ * @param con HTTP-connection to the client
+ * @param rc Original refresh commitment from the previous /melt request
+ * @param[out] result On error, a HTTP-response will be queued and result set accordingly
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
+ */
+static enum GNUNET_GenericReturnValue
+commit_reveal (
+  struct MHD_Connection *con,
+  const struct TALER_RefreshCommitmentP *rc,
+  MHD_RESULT *result)
+{
+  enum GNUNET_DB_QueryStatus qs;
+
+  for (unsigned int retry = 0; retry < 3; retry++)
+  {
+    qs = TEH_plugin->mark_refresh_reveal_success (
+      TEH_plugin->cls,
+      rc);
+    switch (qs)
+    {
+    case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
+      return GNUNET_OK; /* Only happy case */
+    case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
+      *result = TALER_MHD_reply_with_error (con,
+                                            MHD_HTTP_NOT_FOUND,
+                                            TALER_EC_EXCHANGE_REFRESHES_REVEAL_SESSION_UNKNOWN,
+                                            NULL);
+      return GNUNET_SYSERR;
+    case GNUNET_DB_STATUS_HARD_ERROR:
+      *result = TALER_MHD_reply_with_ec (con,
+                                         TALER_EC_GENERIC_DB_STORE_FAILED,
+                                         "mark_refresh_reveal_success");
+      return GNUNET_SYSERR;
+    case GNUNET_DB_STATUS_SOFT_ERROR:
+      break; /* try again */
+    default:
+      GNUNET_break (0);
+      *result = TALER_MHD_reply_with_ec (con,
+                                         TALER_EC_GENERIC_INTERNAL_INVARIANT_FAILURE,
+                                         NULL);
+      return GNUNET_SYSERR;
+    }
+  }
+  /* after unsuccessful retries*/
+  *result = TALER_MHD_reply_with_ec (con,
+                                     TALER_EC_GENERIC_DB_STORE_FAILED,
+                                     "mark_refresh_reveal_success");
+  return GNUNET_SYSERR;
+
+}
+
+
 /**
  * @brief Send a response for "/reveal-melt"
  *
@@ -590,7 +731,7 @@ verify_commitment (
 static MHD_RESULT
 reply_melt_reveal_success (
   struct MHD_Connection *connection,
-  const struct TALER_EXCHANGEDB_Refresh_v27 *refresh)
+  const struct TALER_EXCHANGEDB_Refresh_vDOLDPLUS *refresh)
 {
   json_t *list = json_array ();
   GNUNET_assert (NULL != list);
@@ -622,18 +763,31 @@ TEH_handler_reveal_melt (
   MHD_RESULT result = MHD_NO;
   enum GNUNET_GenericReturnValue ret = GNUNET_SYSERR;
   struct MeltRevealContext actx = {0};
-  struct GNUNET_JSON_Specification tuple[] =     {
+  struct GNUNET_JSON_Specification sig_tuple[] = {
     GNUNET_JSON_spec_fixed_auto (NULL,
                                  &actx.signatures[0]),
     GNUNET_JSON_spec_fixed_auto (NULL,
                                  &actx.signatures[1]),
     GNUNET_JSON_spec_end ()
   };
+  struct GNUNET_JSON_Specification seeds_tuple[] = {
+    GNUNET_JSON_spec_fixed_auto (NULL,
+                                 &actx.transfer_secret_seeds[0]),
+    GNUNET_JSON_spec_fixed_auto (NULL,
+                                 &actx.transfer_secret_seeds[1]),
+    GNUNET_JSON_spec_end ()
+  };
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("rc",
                                  &actx.rc),
-    TALER_JSON_spec_tuple_of ("signatures",
-                              tuple),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_tuple_of ("signatures",
+                                sig_tuple),
+      &actx.no_signatures),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_tuple_of ("batch_seeds",
+                                seeds_tuple),
+      &actx.no_transfer_secret_seeds),
     GNUNET_JSON_spec_mark_optional (
       TALER_JSON_spec_age_commitment ("age_commitment",
                                       &actx.age_commitment),
@@ -646,7 +800,8 @@ TEH_handler_reveal_melt (
    * the size of TALER_CNC_KAPPA.
    * Let's make sure we keep this in sync.
    */
-  _Static_assert (KAPPA_MINUS_1 == 2);
+  _Static_assert (KAPPA_MINUS_1 == 2,
+                  "TALER_CNC_KAPPA isn't 3!?!?");
 
   /* Parse JSON body*/
   ret = TALER_MHD_parse_json_data (rc->connection,
@@ -686,10 +841,21 @@ TEH_handler_reveal_melt (
           actx.no_age_commitment
                 ? NULL
                 : &actx.age_commitment,
-          actx.signatures,
+          actx.no_signatures
+                ? NULL
+                : &actx.signatures,
+          actx.no_transfer_secret_seeds
+                ? NULL
+                : &actx.transfer_secret_seeds,
           &result))
       break;
 
+    if (GNUNET_OK !=
+        commit_reveal (rc->connection,
+                       &actx.rc,
+                       &result))
+      break;
+
     /* Finally, return the signatures */
     result = reply_melt_reveal_success (rc->connection,
                                         &actx.refresh);
