@@ -20,12 +20,12 @@
  */
 #ifndef TALER_EXCHANGE_HTTPD_TERMS_H
 #define TALER_EXCHANGE_HTTPD_TERMS_H
-#include "platform.h"
+#include "taler/platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
 #include <jansson.h>
 #include <microhttpd.h>
-#include "taler_mhd_lib.h"
+#include "taler/taler_mhd_lib.h"
 #include "taler-exchange-httpd_responses.h"
 
 
