@@ -63,6 +63,12 @@ static bool hh_returned_data;
  */
 static bool hh_account_404;
 
+/**
+ * Set to true if the request for history did not
+ * succeed because of some unexpected HTTP request error.
+ */
+static bool hh_error;
+
 /**
  * When did we start the last @e hh request?
  */
@@ -122,6 +128,11 @@ static struct GNUNET_TIME_Relative longpoll_timeout;
  */
 static struct GNUNET_TIME_Relative h404_backoff;
 
+/**
+ * How long do we wait on HTTP history request errors.
+ */
+static struct GNUNET_TIME_Relative hh_error_backoff;
+
 /**
  * Name of our job in the shard table.
  */
@@ -483,7 +494,7 @@ transaction_completed (void)
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  if (! (hh_returned_data || hh_account_404) )
+  if (! (hh_returned_data || hh_account_404 || hh_error) )
   {
     /* Enforce long-polling delay even if the server ignored it
        and returned earlier */
@@ -511,6 +522,16 @@ transaction_completed (void)
   {
     h404_backoff = GNUNET_TIME_UNIT_ZERO;
   }
+  if (hh_error)
+  {
+    hh_error_backoff = GNUNET_TIME_STD_BACKOFF (hh_error_backoff);
+    delayed_until = GNUNET_TIME_relative_to_absolute (
+      hh_error_backoff);
+  }
+  else
+  {
+    hh_error_backoff = GNUNET_TIME_UNIT_ZERO;
+  }
   if (test_mode)
     delayed_until = GNUNET_TIME_UNIT_ZERO_ABS;
   GNUNET_assert (NULL == task);
@@ -818,6 +839,7 @@ history_cb (void *cls,
     }
     break;
   default:
+    hh_error = true;
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Error fetching history: %s (%u)\n",
                 TALER_ErrorCode_get_hint (reply->ec),
@@ -850,6 +872,7 @@ continue_with_shard (void *cls)
   hh_start_time = GNUNET_TIME_absolute_get ();
   hh_returned_data = false;
   hh_account_404 = false;
+  hh_error = false;
   hh = TALER_BANK_credit_history (ctx,
                                   ai->auth,
                                   latest_row_off,
