@@ -35,7 +35,7 @@ PREFIX=
 taler-exchange-dbinit -c test_taler_exchange_httpd.conf &> /dev/null || exit 77
 # Run Exchange HTTPD (in background)
 $PREFIX taler-exchange-httpd -c test_taler_exchange_httpd.conf 2> test-exchange.log &
-
+EPID=$!
 # Give HTTP time to start
 
 for n in `seq 1 100`
@@ -74,8 +74,7 @@ echo -n .
 cat test_taler_exchange_httpd.get | grep -v ^\# | awk '{ print "curl -H \"Accept: */plain\" http://localhost:8081" $1 }' | bash &> /dev/null
 
 echo " DONE"
-# $! is the last backgrounded process, hence the exchange
-kill -TERM $!
+kill -TERM "${EPID}"
 wait $!
 # Return status code from exchange for this script
 exit $?
