@@ -118,9 +118,9 @@ THEN
   END IF;
   -- Well, fine, setup the account
   out_is_wallet
-    = (LOWER (SUBSTRING (in_payto_uri, 22)) =
+    = (LOWER (SUBSTRING (in_payto_uri, 0, 23)) =
        'payto://taler-reserve/') OR
-      (LOWER (SUBSTRING (in_payto_uri, 27)) =
+      (LOWER (SUBSTRING (in_payto_uri, 0, 28)) =
        'payto://taler-reserve-http/');
   INSERT INTO kyc_targets
     (h_normalized_payto
@@ -239,7 +239,7 @@ THEN
     -- Process starts and finishes instantly
     ,in_decision_time
     ,'aml-officer'
-    ,in_decider_pub
+    ,ENCODE(in_decider_pub, 'base64')
     ,TRUE
     )
     RETURNING legitimization_process_serial_id
