@@ -19,8 +19,8 @@ CREATE FUNCTION exchange_do_insert_sanction_list_hit(
   IN in_h_normalized_payto BYTEA,
   IN in_decision_time INT8,
   IN in_expiration_time INT8,
-  IN in_properties TEXT, -- can be NULL
-  IN in_new_rules TEXT, -- can be NULL
+  IN in_properties JSONB, -- can be NULL
+  IN in_new_rules JSONB, -- can be NULL
   IN in_to_investigate BOOLEAN,
   IN in_notify_s TEXT,
   IN ina_events TEXT[],
@@ -78,5 +78,5 @@ EXECUTE FORMAT (
 END $$;
 
 
-COMMENT ON FUNCTION exchange_do_insert_sanction_list_hit(BYTEA, INT8, INT8, TEXT, TEXT, BOOLEAN, TEXT, TEXT[])
+COMMENT ON FUNCTION exchange_do_insert_sanction_list_hit(BYTEA, INT8, INT8, JSONB, JSONB, BOOLEAN, TEXT, TEXT[])
   IS 'Insert result from sanction list check into the table';
