@@ -50,6 +50,10 @@ INSERT INTO kyc_attributes
   ,in_enc_attributes
   ,in_process_row);
 
+-- Wake up taler-exchange-sanctionscheck to check new attributes
+-- This is value for TALER_DBEVENT_EXCHANGE_NEW_KYC_ATTRIBUTES.
+NOTIFY XSX9Z5XGWWYFKXTAYCES63B62527JKNX9XD0131Z08THVV8YW5BZG;
+
 UPDATE legitimization_processes
   SET provider_user_id=in_provider_account_id
      ,provider_legitimization_id=in_provider_legitimization_id
