@@ -1,6 +1,6 @@
 --
 -- This file is part of TALER
--- Copyright (C) 2014--2022 Taler Systems SA
+-- Copyright (C) 2014--2025 Taler Systems SA
 --
 -- TALER is free software; you can redistribute it and/or modify it under the
 -- terms of the GNU General Public License as published by the Free Software
@@ -41,20 +41,20 @@ BEGIN
    WHERE reserve_pub = res_pub;
 
   FOR blind_ev IN
-    SELECT h_blind_ev
-      FROM reserves_out ro
+    SELECT unnest(h_blind_evs)
+      FROM withdraw wd
       JOIN reserve_history rh
-        ON (rh.serial_id = ro.reserve_out_serial_id)
+        ON (rh.serial_id = wd.withdraw_id)
     WHERE rh.reserve_pub = res_pub
-      AND rh.table_name='reserves_out'
+      AND rh.table_name='withdraw'
   LOOP
     SELECT robr.coin_pub
       INTO c_pub
       FROM exchange.recoup_by_reserve robr
-    WHERE robr.reserve_out_serial_id = (
-      SELECT reserve_out_serial_id
-        FROM reserves_out
-      WHERE h_blind_ev = blind_ev
+    WHERE robr.withdraw_id = (
+      SELECT withdraw_id
+        FROM withdraw
+      WHERE h_blind_ev = ANY(h_blind_evs)
     );
     RETURN QUERY
       SELECT kc.denom_sig,
