@@ -86,7 +86,7 @@ TALER_EXCHANGEDB_calculate_transaction_list_totals (
       }
       have_refund = true;
       break;
-    case TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP:
+    case TALER_EXCHANGEDB_TT_RECOUP_REFRESH_RECEIVER:
       /* refunded += pos->value */
       if (0 >
           TALER_amount_add (&refunded,
@@ -97,7 +97,7 @@ TALER_EXCHANGEDB_calculate_transaction_list_totals (
         return GNUNET_SYSERR;
       }
       break;
-    case TALER_EXCHANGEDB_TT_RECOUP:
+    case TALER_EXCHANGEDB_TT_RECOUP_WITHDRAW:
       /* spent += pos->value */
       if (0 >
           TALER_amount_add (&spent,
