@@ -72,7 +72,7 @@ static struct TALER_MerchantWireHashP h_wire_wt;
 
 static struct DenomKeyPair **new_dkp;
 
-static struct TALER_EXCHANGEDB_RefreshRevealedCoin *revealed_coins;
+static struct CoinInfo *revealed_coins;
 
 struct DenomKeyPair
 {
@@ -81,6 +81,16 @@ struct DenomKeyPair
 };
 
 
+struct CoinInfo
+{
+  struct TALER_DenominationHashP h_denom_pub;
+  struct TALER_CoinSpendSignatureP orig_coin_link_sig;
+  struct TALER_BlindedCoinHashP coin_envelope_hash;
+  struct TALER_BlindedDenominationSignature coin_sig;
+  struct TALER_ExchangeBlindingValues exchange_vals;
+  struct TALER_BlindedPlanchet blinded_planchet;
+};
+
 /**
  * Destroy a denomination key pair.  The key is not necessarily removed from the DB.
  *
@@ -229,7 +239,7 @@ run (void *cls)
   struct TALER_EXCHANGEDB_Refund *ref = NULL;
   unsigned int *perm;
   unsigned long long duration_sq;
-  struct TALER_EXCHANGEDB_RefreshRevealedCoin *ccoin;
+  struct CoinInfo *ccoin;
   struct TALER_DenominationPublicKey *new_denom_pubs = NULL;
   struct TALER_DenomFeeSet fees;
   unsigned int count = 0;
@@ -290,7 +300,7 @@ run (void *cls)
                                        struct TALER_DenominationPublicKey);
     revealed_coins
       = GNUNET_new_array (MELT_NEW_COINS,
-                          struct TALER_EXCHANGEDB_RefreshRevealedCoin);
+                          struct CoinInfo);
     for (unsigned int cnt = 0; cnt < MELT_NEW_COINS; cnt++)
     {
       struct GNUNET_TIME_Timestamp now;
