@@ -21,9 +21,9 @@
 #ifndef PG_BEGIN_SHARD_H
 #define PG_BEGIN_SHARD_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 /**
  * Function called to grab a work shard on an operation @a op. Runs in its
  * own transaction.
