@@ -30,7 +30,7 @@
 enum GNUNET_DB_QueryStatus
 TEH_PG_do_refresh (
   void *cls,
-  struct TALER_EXCHANGEDB_Refresh_v27 *refresh,
+  struct TALER_EXCHANGEDB_Refresh_vDOLDPLUS *refresh,
   const struct GNUNET_TIME_Timestamp *timestamp,
   bool *found,
   uint32_t *noreveal_index,
@@ -43,9 +43,14 @@ TEH_PG_do_refresh (
   struct GNUNET_PQ_QueryParam params[] = {
     GNUNET_PQ_query_param_auto_from_type (&refresh->rc),
     GNUNET_PQ_query_param_timestamp (timestamp),
-    GNUNET_PQ_query_param_auto_from_type (&refresh->refresh_seed),
+    GNUNET_PQ_query_param_auto_from_type (&refresh->refresh_seed), /* 3 */
+    (refresh->is_v27_refresh)
+        ? GNUNET_PQ_query_param_null ()
+        : GNUNET_PQ_query_param_array_auto_from_type (refresh->num_coins,
+                                                      refresh->transfer_pubs,
+                                                      pg->conn),
     GNUNET_PQ_query_param_auto_from_type (&refresh->planchets_h),
-    TALER_PQ_query_param_amount (pg->conn,
+    TALER_PQ_query_param_amount (pg->conn, /* 6 */
                                  &refresh->amount_with_fee),
     (refresh->no_blinding_seed)
         ? GNUNET_PQ_query_param_null ()
@@ -56,18 +61,18 @@ TEH_PG_do_refresh (
                                                pg->conn)
         : GNUNET_PQ_query_param_null (),
     (0 < refresh->num_cs_r_values)
-        ? GNUNET_PQ_query_param_uint64 (&refresh->cs_r_choices)
+        ? GNUNET_PQ_query_param_uint64 (&refresh->cs_r_choices) /* 9 */
         : GNUNET_PQ_query_param_null (),
     GNUNET_PQ_query_param_auto_from_type (&refresh->selected_h),
     TALER_PQ_query_param_array_blinded_denom_sig (refresh->num_coins,
                                                   refresh->denom_sigs,
                                                   pg->conn),
-    GNUNET_PQ_query_param_array_uint64 (refresh->num_coins,
+    GNUNET_PQ_query_param_array_uint64 (refresh->num_coins, /* 12 */
                                         refresh->denom_serials,
                                         pg->conn),
     GNUNET_PQ_query_param_auto_from_type (&refresh->coin.coin_pub),
     GNUNET_PQ_query_param_auto_from_type (&refresh->coin_sig),
-    GNUNET_PQ_query_param_uint32 (&refresh->noreveal_index),
+    GNUNET_PQ_query_param_uint32 (&refresh->noreveal_index), /* 15 */
     GNUNET_PQ_query_param_bool (*zombie_required),
     GNUNET_PQ_query_param_end
   };
@@ -108,7 +113,7 @@ TEH_PG_do_refresh (
            ",out_noreveal_index AS noreveal_index"
            ",out_coin_balance AS coin_balance"
            " FROM exchange_do_refresh"
-           " ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15);");
+           " ($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14,$15,$16);");
   qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
                                                  "call_refresh",
                                                  params,
