@@ -45,7 +45,7 @@
 enum GNUNET_DB_QueryStatus
 TEH_PG_do_refresh (
   void *cls,
-  struct TALER_EXCHANGEDB_Refresh_v27 *refresh,
+  struct TALER_EXCHANGEDB_Refresh_vDOLDPLUS *refresh,
   const struct GNUNET_TIME_Timestamp *timestamp,
   bool *found,
   uint32_t *noreveal_index,
