@@ -1,6 +1,6 @@
 /*
    This file is part of TALER
-   Copyright (C) 2022 Taler Systems SA
+   Copyright (C) 2022, 2024 Taler Systems SA
 
    TALER is free software; you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
@@ -18,10 +18,10 @@
  * @brief Implementation of the drain_kyc_alert function for Postgres
  * @author Christian Grothoff
  */
-#include "platform.h"
-#include "taler_error_codes.h"
-#include "taler_dbevents.h"
-#include "taler_pq_lib.h"
+#include "taler/platform.h"
+#include "taler/taler_error_codes.h"
+#include "taler/taler_dbevents.h"
+#include "taler/taler_pq_lib.h"
 #include "pg_drain_kyc_alert.h"
 #include "pg_helper.h"
 
@@ -29,7 +29,7 @@
 enum GNUNET_DB_QueryStatus
 TEH_PG_drain_kyc_alert (void *cls,
                         uint32_t trigger_type,
-                        struct TALER_PaytoHashP *h_payto)
+                        struct TALER_NormalizedPaytoHashP *h_payto)
 {
   struct PostgresClosure *pg = cls;
   struct GNUNET_PQ_QueryParam params[] = {
