@@ -21,9 +21,9 @@
 #ifndef PG_EVENT_LISTEN_H
 #define PG_EVENT_LISTEN_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 /**
  * Register callback to be invoked on events of type @a es.
  *
