@@ -489,7 +489,7 @@ add_old_coin_recoup (void *cls,
     }
     tl = GNUNET_new (struct TALER_EXCHANGEDB_TransactionList);
     tl->next = chc->head;
-    tl->type = TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP;
+    tl->type = TALER_EXCHANGEDB_TT_RECOUP_REFRESH_RECEIVER;
     tl->details.old_coin_recoup = recoup;
     tl->serial_id = serial_id;
     tl->coin_history_id = chc->chid;
@@ -553,7 +553,7 @@ add_coin_recoup (void *cls,
     }
     tl = GNUNET_new (struct TALER_EXCHANGEDB_TransactionList);
     tl->next = chc->head;
-    tl->type = TALER_EXCHANGEDB_TT_RECOUP;
+    tl->type = TALER_EXCHANGEDB_TT_RECOUP_WITHDRAW;
     tl->details.recoup = recoup;
     tl->serial_id = serial_id;
     tl->coin_history_id = chc->chid;
@@ -745,11 +745,7 @@ handle_history_entry (void *cls,
     { "refunds",
       "get_refunds_by_coin",
       &add_coin_refund },
-    [TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP] =
-    { "recoup_refresh::OLD",
-      "recoup_by_old_coin",
-      &add_old_coin_recoup },
-    [TALER_EXCHANGEDB_TT_RECOUP] =
+    [TALER_EXCHANGEDB_TT_RECOUP_WITHDRAW] =
     { "recoup",
       "recoup_by_coin",
       &add_coin_recoup },
@@ -757,6 +753,10 @@ handle_history_entry (void *cls,
     { "recoup_refresh::NEW",
       "recoup_by_refreshed_coin",
       &add_coin_recoup_refresh },
+    [TALER_EXCHANGEDB_TT_RECOUP_REFRESH_RECEIVER] =
+    { "recoup_refresh::OLD",
+      "recoup_by_old_coin",
+      &add_old_coin_recoup },
     [TALER_EXCHANGEDB_TT_RESERVE_OPEN] =
     { "reserves_open_deposits",
       "reserve_open_by_coin",
