@@ -29,7 +29,7 @@
 enum GNUNET_DB_QueryStatus
 TEH_PG_get_refresh (void *cls,
                     const struct TALER_RefreshCommitmentP *rc,
-                    struct TALER_EXCHANGEDB_Refresh_v27 *refresh)
+                    struct TALER_EXCHANGEDB_Refresh_vDOLDPLUS *refresh)
 {
   struct PostgresClosure *pg = cls;
   struct GNUNET_PQ_QueryParam params[] = {
@@ -38,9 +38,12 @@ TEH_PG_get_refresh (void *cls,
   };
   bool no_cs_r_values;
   bool no_cs_r_choices;
+  bool no_transfer_pubs;
   size_t num_denom_sigs;
+  size_t num_transfer_pubs;
   struct TALER_BlindedDenominationSignature *denom_sigs = NULL;
   struct GNUNET_CRYPTO_CSPublicRPairP *cs_r_values = NULL;
+  struct TALER_TransferPublicKeyP *transfer_pubs = NULL;
   uint64_t *denom_serials = NULL;
   struct GNUNET_PQ_ResultSpec rs[] = {
     TALER_PQ_RESULT_SPEC_AMOUNT ("amount_with_fee",
@@ -75,6 +78,13 @@ TEH_PG_get_refresh (void *cls,
                                           &refresh->planchets_h),
     GNUNET_PQ_result_spec_auto_from_type ("selected_h",
                                           &refresh->selected_h),
+    GNUNET_PQ_result_spec_allow_null (
+      GNUNET_PQ_result_spec_array_fixed_size (pg->conn,
+                                              "transfer_pubs",
+                                              sizeof(*transfer_pubs),
+                                              &num_transfer_pubs,
+                                              (void **) &transfer_pubs),
+      &no_transfer_pubs),
     GNUNET_PQ_result_spec_array_uint64 (pg->conn,
                                         "denom_serials",
                                         &refresh->num_coins,
@@ -83,6 +93,8 @@ TEH_PG_get_refresh (void *cls,
                                                   "denom_sigs",
                                                   &num_denom_sigs,
                                                   &denom_sigs),
+    GNUNET_PQ_result_spec_bool ("revealed",
+                                &refresh->revealed),
     GNUNET_PQ_result_spec_end
   };
   enum GNUNET_DB_QueryStatus qs;
@@ -103,9 +115,11 @@ TEH_PG_get_refresh (void *cls,
            ",cs_r_values"
            ",cs_r_choices"
            ",planchets_h"
+           ",transfer_pubs"
            ",selected_h"
            ",denom_serials"
            ",denom_sigs"
+           ",revealed"
            " FROM refresh"
            " JOIN known_coins kc"
            " ON (old_coin_pub = kc.coin_pub)"
@@ -138,6 +152,27 @@ TEH_PG_get_refresh (void *cls,
     GNUNET_PQ_cleanup_result (rs);
     return GNUNET_DB_STATUS_HARD_ERROR;
   }
+  if (no_transfer_pubs)
+  {
+    refresh->is_v27_refresh = true;
+    refresh->transfer_pubs = NULL;
+  }
+  else
+  {
+    if (num_transfer_pubs != refresh->num_coins)
+    {
+      GNUNET_break (0);
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "got inconsistent number of transfer_pubs in refresh from DB: "
+                  "num_coins=%ld, num_transfer_pubs=%ld\n",
+                  refresh->num_coins,
+                  num_transfer_pubs);
+      GNUNET_PQ_cleanup_result (rs);
+      return GNUNET_DB_STATUS_HARD_ERROR;
+    }
+    refresh->is_v27_refresh = false;
+    refresh->transfer_pubs = transfer_pubs;
+  }
   if (refresh->no_blinding_seed != no_cs_r_values)
   {
     GNUNET_break (0);
@@ -164,6 +199,7 @@ TEH_PG_get_refresh (void *cls,
   refresh->denom_sigs = denom_sigs;
   refresh->denom_serials = denom_serials;
   refresh->cs_r_values = cs_r_values;
+  transfer_pubs = NULL;
   denom_sigs = NULL;
   denom_serials = NULL;
   cs_r_values = NULL;
