@@ -37,6 +37,6 @@
 enum GNUNET_DB_QueryStatus
 TEH_PG_get_refresh (void *cls,
                     const struct TALER_RefreshCommitmentP *rc,
-                    struct TALER_EXCHANGEDB_Refresh_v27 *refresh);
+                    struct TALER_EXCHANGEDB_Refresh_vDOLDPLUS *refresh);
 
 #endif
