@@ -151,8 +151,8 @@ TEH_PG_get_wire_accounts (void *cls,
            "SELECT"
            " payto_uri"
            ",conversion_url"
-           ",debit_restrictions"
-           ",credit_restrictions"
+           ",debit_restrictions::TEXT"
+           ",credit_restrictions::TEXT"
            ",master_sig"
            ",bank_label"
            ",priority"
