@@ -21,9 +21,9 @@
 #ifndef PG_INSERT_PARTNER_H
 #define PG_INSERT_PARTNER_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 /**
  * Function called to store configuration data about a partner
  * exchange that we are federated with.
