@@ -78,7 +78,7 @@ TEH_PG_insert_sanction_list_hit (
            "SELECT"
            " out_outcome_serial_id AS outcome_serial_id"
            " FROM exchange_do_insert_sanction_list_hit"
-           "($1,$2,$3,$4,$5,$6,$7,$8);");
+           "($1,$2,$3,$4::TEXT::JSONB,$5::TEXT::JSONB,$6,$7,$8);");
   qs = GNUNET_PQ_eval_prepared_singleton_select (
     pg->conn,
     "do_insert_sanction_list_hit",
