@@ -21,9 +21,9 @@
 #ifndef PG_INSERT_SUCCESSOR_MEASURE_H
 #define PG_INSERT_SUCCESSOR_MEASURE_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 
 enum GNUNET_DB_QueryStatus
 TEH_PG_insert_successor_measure (
