@@ -32,6 +32,8 @@
  *
  * @param cls closure
  * @param h_payto hash of account to lookup history for
+ * @param offset row ID to start returning results from
+ * @param limit how many results to return, negative for descending order
  * @param cb function to call on results
  * @param cb_cls closure for @a cb
  * @return database transaction status
@@ -40,6 +42,8 @@ enum GNUNET_DB_QueryStatus
 TEH_PG_lookup_aml_history (
   void *cls,
   const struct TALER_NormalizedPaytoHashP *h_payto,
+  uint64_t offset,
+  int64_t limit,
   TALER_EXCHANGEDB_AmlHistoryCallback cb,
   void *cb_cls);
 
