@@ -21,9 +21,9 @@
 #ifndef PG_LOOKUP_GLOBAL_FEE_BY_TIME_H
 #define PG_LOOKUP_GLOBAL_FEE_BY_TIME_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 
 /**
  * Lookup information about known global fees.
