@@ -63,7 +63,7 @@ TEH_PG_lookup_pending_legitimization (
   PREPARE (pg,
            "lookup_pending_legitimization",
            "SELECT "
-           " lm.jmeasures"
+           " lm.jmeasures::TEXT"
            ",kt.h_normalized_payto"
            ",kt.is_wallet"
            ",lm.access_token"
