@@ -18,10 +18,10 @@
  * @brief Low-level (statement-level) Postgres database access for the exchange
  * @author Christian Grothoff
  */
-#include "platform.h"
-#include "taler_error_codes.h"
-#include "taler_dbevents.h"
-#include "taler_pq_lib.h"
+#include "taler/platform.h"
+#include "taler/taler_error_codes.h"
+#include "taler/taler_dbevents.h"
+#include "taler/taler_pq_lib.h"
 #include "pg_lookup_serial_by_table.h"
 #include "pg_helper.h"
 
@@ -77,6 +77,14 @@ TEH_PG_lookup_serial_by_table (void *cls,
               " ORDER BY wire_target_serial_id DESC"
               " LIMIT 1;");
     break;
+  case TALER_EXCHANGEDB_RT_KYC_TARGETS:
+    XPREPARE ("select_serial_by_table_kyc_targets",
+              "SELECT"
+              " kyc_target_serial_id AS serial"
+              " FROM kyc_targets"
+              " ORDER BY kyc_target_serial_id DESC"
+              " LIMIT 1;");
+    break;
   case TALER_EXCHANGEDB_RT_RESERVES:
     XPREPARE ("select_serial_by_table_reserves",
               "SELECT"
@@ -125,14 +133,6 @@ TEH_PG_lookup_serial_by_table (void *cls,
               " ORDER BY reserve_open_deposit_uuid DESC"
               " LIMIT 1;");
     break;
-  case TALER_EXCHANGEDB_RT_RESERVES_OUT:
-    XPREPARE ("select_serial_by_table_reserves_out",
-              "SELECT"
-              " reserve_out_serial_id AS serial"
-              " FROM reserves_out"
-              " ORDER BY reserve_out_serial_id DESC"
-              " LIMIT 1;");
-    break;
   case TALER_EXCHANGEDB_RT_AUDITORS:
     XPREPARE ("select_serial_by_table_auditors",
               "SELECT"
