@@ -21,9 +21,9 @@
 #ifndef PG_LOOKUP_SIGNING_KEY_H
 #define PG_LOOKUP_SIGNING_KEY_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 
 
 /**
@@ -39,4 +39,5 @@ TEH_PG_lookup_signing_key (
   void *cls,
   const struct TALER_ExchangePublicKeyP *exchange_pub,
   struct TALER_EXCHANGEDB_SignkeyMetaData *meta);
+
 #endif
