@@ -21,9 +21,9 @@
 #ifndef PG_PROFIT_DRAINS_GET_PENDING_H
 #define PG_PROFIT_DRAINS_GET_PENDING_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 
 /**
  * Get profit drain operation ready to execute.
