@@ -21,9 +21,9 @@
 #ifndef PG_RESERVES_GET_H
 #define PG_RESERVES_GET_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 /**
  * Get the summary of a reserve.
  *
