@@ -21,9 +21,9 @@
 #ifndef PG_RESERVES_UPDATE_H
 #define PG_RESERVES_UPDATE_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 
 /**
  * Updates a reserve with the data from the given reserve structure.
