@@ -21,9 +21,9 @@
 #ifndef PG_ROLLBACK_H
 #define PG_ROLLBACK_H
 
-#include "taler_util.h"
-#include "taler_json_lib.h"
-#include "taler_exchangedb_plugin.h"
+#include "taler/taler_util.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchangedb_plugin.h"
 
 /**
  * Roll back the current transaction of a database connection.
