@@ -18,10 +18,10 @@
  * @brief Implementation of the select_batch_deposits_missing_wire function for Postgres
  * @author Christian Grothoff
  */
-#include "platform.h"
-#include "taler_error_codes.h"
-#include "taler_dbevents.h"
-#include "taler_pq_lib.h"
+#include "taler/platform.h"
+#include "taler/taler_error_codes.h"
+#include "taler/taler_dbevents.h"
+#include "taler/taler_pq_lib.h"
 #include "pg_select_batch_deposits_missing_wire.h"
 #include "pg_helper.h"
 
