@@ -134,7 +134,7 @@ TEH_COMMON_free_coin_transaction_list (
     case TALER_EXCHANGEDB_TT_MELT:
       GNUNET_free (tl->details.melt);
       break;
-    case TALER_EXCHANGEDB_TT_OLD_COIN_RECOUP:
+    case TALER_EXCHANGEDB_TT_RECOUP_REFRESH_RECEIVER:
       {
         struct TALER_EXCHANGEDB_RecoupRefreshListEntry *rr;
 
@@ -146,7 +146,7 @@ TEH_COMMON_free_coin_transaction_list (
     case TALER_EXCHANGEDB_TT_REFUND:
       GNUNET_free (tl->details.refund);
       break;
-    case TALER_EXCHANGEDB_TT_RECOUP:
+    case TALER_EXCHANGEDB_TT_RECOUP_WITHDRAW:
       GNUNET_free (tl->details.recoup);
       break;
     case TALER_EXCHANGEDB_TT_RECOUP_REFRESH:
