@@ -48,7 +48,6 @@
 #include "pg_do_withdraw.h"
 #include "pg_do_check_deposit_idempotent.h"
 #include "pg_do_deposit.h"
-#include "pg_do_melt.h"
 #include "pg_do_purse_delete.h"
 #include "pg_do_purse_deposit.h"
 #include "pg_do_purse_merge.h"
@@ -88,7 +87,6 @@
 #include "pg_get_purse_request.h"
 #include "pg_get_ready_deposit.h"
 #include "pg_get_refresh.h"
-#include "pg_get_refresh_reveal.h"
 #include "pg_get_reserve_balance.h"
 #include "pg_get_reserve_by_h_planchets.h"
 #include "pg_get_reserve_history.h"
@@ -159,6 +157,7 @@
 #include "pg_lookup_wire_fee_by_time.h"
 #include "pg_lookup_wire_timestamp.h"
 #include "pg_lookup_wire_transfer.h"
+#include "pg_mark_refresh_reveal_success.h"
 #include "pg_persist_kyc_attributes.h"
 #include "pg_persist_policy_details.h"
 #include "pg_preflight.h"
@@ -558,6 +557,8 @@ libtaler_plugin_exchangedb_postgres_init (void *cls)
     = &TEH_PG_add_policy_fulfillment_proof;
   plugin->do_refresh
     = &TEH_PG_do_refresh;
+  plugin->mark_refresh_reveal_success
+    = &TEH_PG_mark_refresh_reveal_success;
   plugin->do_refund
     = &TEH_PG_do_refund;
   plugin->do_recoup
