@@ -18,11 +18,11 @@
  * @brief Utility functions for extensions
  * @author Özgür Kesim
  */
-#include "platform.h"
-#include "taler_extensions_policy.h"
-#include "taler_util.h"
-#include "taler_signatures.h"
-#include "taler_extensions.h"
+#include "taler/platform.h"
+#include "taler/taler_extensions_policy.h"
+#include "taler/taler_util.h"
+#include "taler/taler_signatures.h"
+#include "taler/taler_extensions.h"
 #include "stdint.h"
 
 /* head of the list of all registered extensions */
