@@ -68,6 +68,15 @@ struct TALER_BANK_AuthenticationData
    */
   char *wire_gateway_url;
 
+  /**
+   * Base URL including "/accpunts/$USERNAME/" to use
+   * to talk to the core bank API. Useful to get a more
+   * specific access token instead of using basic authentication
+   * the whole time. Optional, can be NULL (as we do not
+   * require the core bank API to actually always be available).
+   */
+  char *core_bank_url;
+
   /**
    * Which authentication method should we use?
    */
