@@ -41,9 +41,9 @@
  * @deprecated
  */
 #define TALER_json_warn(error)                                         \
-  GNUNET_log (GNUNET_ERROR_TYPE_WARNING,                                \
-              "JSON parsing failed at %s:%u: %s (%s)\n",                  \
-              __FILE__, __LINE__, error.text, error.source)
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,                                \
+                    "JSON parsing failed at %s:%u: %s (%s)\n",                  \
+                    __FILE__, __LINE__, error.text, error.source)
 
 
 /**
@@ -67,8 +67,8 @@ TALER_JSON_pack_time_abs_human (
  * @param ec error code to encode using canonical field names
  */
 #define TALER_JSON_pack_ec(ec) \
-  GNUNET_JSON_pack_string ("hint", TALER_ErrorCode_get_hint (ec)), \
-  GNUNET_JSON_pack_uint64 ("code", ec)
+        GNUNET_JSON_pack_string ("hint", TALER_ErrorCode_get_hint (ec)), \
+        GNUNET_JSON_pack_uint64 ("code", ec)
 
 
 /**
@@ -341,6 +341,22 @@ TALER_JSON_pack_array_of_data (
   const void *data,
   size_t size);
 
+/**
+ * Generate packer for an array of data of the same size,
+ * encoded in Crockford base32-encoding.
+ * The size is taken via sizeof().
+ *
+ * @param name name of the field to add to the object
+ * @param num number of elements in the @a array
+ * @param data pointer to the list of elements
+ * @return json pack specification
+ */
+#define TALER_JSON_pack_array_of_data_auto(name,num,data) \
+        TALER_JSON_pack_array_of_data ((name), \
+                                       (num), \
+                                       (data), \
+                                       sizeof(*(data)))
+
 
 /******************  Specs For Parsing JSON  *********************/
 
@@ -481,10 +497,10 @@ TALER_JSON_spec_kycte (const char *name,
  * @param[out] dfs a `struct TALER_DenomFeeSet` to initialize
  */
 #define TALER_JSON_SPEC_DENOM_FEES(pfx,currency,dfs) \
-  TALER_JSON_spec_amount (pfx "_withdraw", (currency), &(dfs)->withdraw), \
-  TALER_JSON_spec_amount (pfx "_deposit", (currency), &(dfs)->deposit),   \
-  TALER_JSON_spec_amount (pfx "_refresh", (currency), &(dfs)->refresh),   \
-  TALER_JSON_spec_amount (pfx "_refund", (currency), &(dfs)->refund)
+        TALER_JSON_spec_amount (pfx "_withdraw", (currency), &(dfs)->withdraw), \
+        TALER_JSON_spec_amount (pfx "_deposit", (currency), &(dfs)->deposit),   \
+        TALER_JSON_spec_amount (pfx "_refresh", (currency), &(dfs)->refresh),   \
+        TALER_JSON_spec_amount (pfx "_refund", (currency), &(dfs)->refund)
 
 
 /**
@@ -495,10 +511,10 @@ TALER_JSON_spec_kycte (const char *name,
  * @param dfs a `struct TALER_DenomFeeSet` to pack
  */
 #define TALER_JSON_PACK_DENOM_FEES(pfx, dfs) \
-  TALER_JSON_pack_amount (pfx "_withdraw", &(dfs)->withdraw),   \
-  TALER_JSON_pack_amount (pfx "_deposit", &(dfs)->deposit),     \
-  TALER_JSON_pack_amount (pfx "_refresh", &(dfs)->refresh),     \
-  TALER_JSON_pack_amount (pfx "_refund", &(dfs)->refund)
+        TALER_JSON_pack_amount (pfx "_withdraw", &(dfs)->withdraw),   \
+        TALER_JSON_pack_amount (pfx "_deposit", &(dfs)->deposit),     \
+        TALER_JSON_pack_amount (pfx "_refresh", &(dfs)->refresh),     \
+        TALER_JSON_pack_amount (pfx "_refund", &(dfs)->refund)
 
 
 /**
@@ -508,9 +524,9 @@ TALER_JSON_spec_kycte (const char *name,
  * @param[out] gfs a `struct TALER_GlobalFeeSet` to initialize
  */
 #define TALER_JSON_SPEC_GLOBAL_FEES(currency,gfs) \
-  TALER_JSON_spec_amount ("history_fee", (currency), &(gfs)->history),   \
-  TALER_JSON_spec_amount ("account_fee", (currency), &(gfs)->account),   \
-  TALER_JSON_spec_amount ("purse_fee", (currency), &(gfs)->purse)
+        TALER_JSON_spec_amount ("history_fee", (currency), &(gfs)->history),   \
+        TALER_JSON_spec_amount ("account_fee", (currency), &(gfs)->account),   \
+        TALER_JSON_spec_amount ("purse_fee", (currency), &(gfs)->purse)
 
 /**
  * Macro to pack all of the global fees.
@@ -518,9 +534,9 @@ TALER_JSON_spec_kycte (const char *name,
  * @param gfs a `struct TALER_GlobalFeeSet` to pack
  */
 #define TALER_JSON_PACK_GLOBAL_FEES(gfs) \
-  TALER_JSON_pack_amount ("history_fee", &(gfs)->history),     \
-  TALER_JSON_pack_amount ("account_fee", &(gfs)->account),     \
-  TALER_JSON_pack_amount ("purse_fee", &(gfs)->purse)
+        TALER_JSON_pack_amount ("history_fee", &(gfs)->history),     \
+        TALER_JSON_pack_amount ("account_fee", &(gfs)->account),     \
+        TALER_JSON_pack_amount ("purse_fee", &(gfs)->purse)
 
 
 /**
