@@ -2803,7 +2803,7 @@ TALER_TESTING_get_trait (const struct TALER_TESTING_Trait *traits,
         op (blinding_seed, const struct TALER_BlindingMasterSeedP)       \
         op (withdraw_commitment, const struct TALER_HashBlindedPlanchetsP)       \
         op (kappa_seed, const struct TALER_KappaWithdrawMasterSeedP)     \
-        op (refresh_secret, const struct TALER_RefreshMasterSecretP)     \
+        op (refresh_seed, const struct TALER_PublicRefreshMasterSeedP)     \
         op (reserve_pub, const struct TALER_ReservePublicKeyP)           \
         op (merchant_priv, const struct TALER_MerchantPrivateKeyP)       \
         op (merchant_pub, const struct TALER_MerchantPublicKeyP)         \
