@@ -171,8 +171,6 @@ parse_amount_any_array (void *cls,
   struct AmountArrayCtx *ctx = cls;
   struct TALER_Amount **out = spec->ptr;
 
-  GNUNET_assert (NULL != ctx);
-  GNUNET_assert (NULL != out);
   *out = NULL;
   if (NULL != ctx->len)
     *ctx->len = 0;
@@ -249,6 +247,8 @@ TALER_JSON_spec_amount_any_array (const char *field,
 
   GNUNET_assert (NULL != amounts_len);
   GNUNET_assert (NULL != amounts);
+  *amounts = NULL;
+  *amounts_len = 0;
   ctx = GNUNET_new (struct AmountArrayCtx);
   ctx->len = amounts_len;
   {
@@ -2162,7 +2162,8 @@ TALER_JSON_spec_array_of_data (
   struct closure_array_of_data *cls;
 
   GNUNET_assert (0< entry_size);
-  GNUNET_assert (NULL != entries);
+  *entries = NULL;
+  *num_entries = 0;
   cls = GNUNET_new (struct closure_array_of_data);
   cls->num_entries = num_entries;
   cls->entry_size = entry_size;
@@ -2186,6 +2187,8 @@ TALER_JSON_spec_array_of_denom_pub_h (
   size_t *num_entries,
   struct TALER_DenominationHashP **entries)
 {
+  *num_entries = 0;
+  *entries = NULL;
   return TALER_JSON_spec_array_of_data (
     field,
     sizeof (struct TALER_DenominationHashP),
@@ -2253,6 +2256,8 @@ TALER_JSON_spec_array_of_blinded_denom_sigs (
   size_t num_entries,
   struct TALER_BlindedDenominationSignature *entries)
 {
+  for (size_t i = 0; i<num_entries; i++)
+    entries[i].blinded_sig = NULL;
   struct GNUNET_JSON_Specification ret = {
     .parser = &parse_array_of_blinded_denom_sigs,
     .ptr = entries,
