@@ -58,6 +58,55 @@ test_amount (void)
 }
 
 
+/**
+ * Verify JSON packing/parsing for amount arrays.
+ *
+ * @return 0 on success
+ */
+static int
+test_amount_array (void)
+{
+  struct TALER_Amount amounts[2];
+  struct TALER_Amount *parsed = NULL;
+  size_t parsed_len = 0;
+  struct GNUNET_JSON_Specification spec[2];
+  json_t *doc;
+  const size_t num_amounts = sizeof (amounts) / sizeof (amounts[0]);
+
+  GNUNET_assert (GNUNET_OK ==
+                 TALER_string_to_amount ("EUR:1.2",
+                                         &amounts[0]));
+  GNUNET_assert (GNUNET_OK ==
+                 TALER_string_to_amount ("EUR:3.4",
+                                         &amounts[1]));
+
+  spec[0] = TALER_JSON_spec_amount_any_array ("amounts",
+                                              &parsed_len,
+                                              &parsed);
+  spec[1] = GNUNET_JSON_spec_end ();
+
+  doc = GNUNET_JSON_PACK (
+    TALER_JSON_pack_amount_array ("amounts",
+                                  num_amounts,
+                                  amounts));
+  GNUNET_assert (NULL != doc);
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_JSON_parse (doc,
+                                    spec,
+                                    NULL,
+                                    NULL));
+  GNUNET_assert (parsed_len == num_amounts);
+  for (size_t i = 0; i<num_amounts; i++)
+    GNUNET_assert (0 ==
+                   TALER_amount_cmp (&amounts[i],
+                                     &parsed[i]));
+  GNUNET_JSON_parse_free (spec);
+  json_decref (doc);
+
+  return 0;
+}
+
+
 struct TestPath_Closure
 {
   const char **object_ids;
@@ -474,6 +523,8 @@ main (int argc,
                     NULL);
   if (0 != test_amount ())
     return 1;
+  if (0 != test_amount_array ())
+    return 1;
   if (0 != test_contract ())
     return 2;
   if (0 != test_json_canon ())
