@@ -353,6 +353,11 @@ struct TALER_KYCLOGIC_WebhookHandle
    */
   uint64_t process_row;
 
+  /**
+   * HTTP status returned by Persona to us.
+   */
+  unsigned int persona_http_status;
+
   /**
    * HTTP response code to return asynchronously.
    */
@@ -1635,6 +1640,38 @@ webhook_post_conversion_cb (void *cls,
   struct TALER_KYCLOGIC_WebhookHandle *wh = cls;
 
   wh->ec = NULL;
+  if (! json_is_string (json_object_get (attr,
+                                         "FORM_ID")))
+  {
+    struct MHD_Response *resp;
+
+    /* Failure in our helper */
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Mandatory FORM_ID not set in result\n");
+    json_dumpf (attr,
+                stderr,
+                JSON_INDENT (2));
+    resp = TALER_MHD_MAKE_JSON_PACK (
+      GNUNET_JSON_pack_uint64 ("persona_http_status",
+                               wh->persona_http_status),
+      GNUNET_JSON_pack_object_incref ("persona_body",
+                                      (json_t *) attr));
+    wh->cb (wh->cb_cls,
+            wh->process_row,
+            &wh->h_payto,
+            wh->is_wallet,
+            wh->pd->section,
+            NULL,
+            wh->inquiry_id,
+            TALER_KYCLOGIC_STATUS_PROVIDER_FAILED,
+            GNUNET_TIME_UNIT_ZERO_ABS, /* expiration */
+            NULL,
+            MHD_HTTP_BAD_GATEWAY,
+            resp);
+    persona_webhook_cancel (wh);
+    return;
+  }
+
   webhook_generic_reply (wh,
                          TALER_KYCLOGIC_STATUS_SUCCESS,
                          wh->account_id,
@@ -1663,6 +1700,7 @@ handle_webhook_finished (void *cls,
                                         "data");
 
   wh->job = NULL;
+  wh->persona_http_status = response_code;
   switch (response_code)
   {
   case MHD_HTTP_OK:
