@@ -23,7 +23,7 @@ FULLNAME=$(echo "$J" | jq -r '[.first_name,.last_name]|join(" ")')
 echo "$J" \
     | jq \
           --arg full_name "${FULLNAME}" \
-          '{FULL_NAME: $full_name, "phone": .phone, "id": .id}' \
+          '{"FORM_ID": "oauth2-nda-form","FULL_NAME": $full_name, "phone": .phone, "id": .id}' \
     | jq \
           'del(..|select(.==null))'
 
