@@ -26,7 +26,7 @@ FULLNAME=$(echo "$J" | jq -r '[.first,.last]|join(" ")')
 echo "$J" \
     | jq \
           --arg full_name "${FULLNAME}" \
-          '{FORM_ID:"test",FULL_NAME: $full_name,"DATE_OF_BIRTH":.birthdate,"id":.id,}' \
+          '{"FORM_ID":"test","FULL_NAME": $full_name,"DATE_OF_BIRTH":.birthdate,"id":.id,}' \
     | jq \
           'del(..|select(.==null))'
 exit 0
