@@ -19,8 +19,8 @@
  * @brief curl easy handle defaults
  * @author Florian Dold
  */
-#include "platform.h"
-#include "taler_curl_lib.h"
+#include "taler/platform.h"
+#include "taler/taler_curl_lib.h"
 #include "auditor_api_curl_defaults.h"
 
 
