@@ -19,17 +19,17 @@
  * @brief Implementation of the /deposit request of the auditor's HTTP API
  * @author Christian Grothoff
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <jansson.h>
 #include <microhttpd.h> /* just for HTTP status codes */
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
-#include "taler_util.h"
-#include "taler_curl_lib.h"
-#include "taler_json_lib.h"
-#include "taler_auditor_service.h"
-#include "taler_signatures.h"
+#include "taler/taler_util.h"
+#include "taler/taler_curl_lib.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_auditor_service.h"
+#include "taler/taler_signatures.h"
 #include "auditor_api_curl_defaults.h"
 
 
