@@ -346,10 +346,15 @@ TALER_EXCHANGE_kyc_check (
         job_headers,
         "Content-Type: application/json");
   {
+    union TALER_AccountPublicKeyP account_pub;
     union TALER_AccountSignatureP account_sig;
     char *sig_hdr;
+    char *pub_hdr;
     char *hdr;
 
+    GNUNET_CRYPTO_eddsa_key_get_public (
+      &account_priv->reserve_priv.eddsa_priv,
+      &account_pub.reserve_pub.eddsa_pub);
     TALER_account_kyc_auth_sign (account_priv,
                                  &account_sig);
 
@@ -364,6 +369,18 @@ TALER_EXCHANGE_kyc_check (
     job_headers = curl_slist_append (job_headers,
                                      hdr);
     GNUNET_free (hdr);
+
+    pub_hdr = GNUNET_STRINGS_data_to_string_alloc (
+      &account_pub,
+      sizeof (account_pub));
+    GNUNET_asprintf (&hdr,
+                     "%s: %s",
+                     TALER_HTTP_HEADER_ACCOUNT_OWNER_PUBKEY,
+                     pub_hdr);
+    GNUNET_free (pub_hdr);
+    job_headers = curl_slist_append (job_headers,
+                                     hdr);
+    GNUNET_free (hdr);
     if (NULL == job_headers)
     {
       GNUNET_break (0);
