@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2015-2023 Taler Systems SA
+  Copyright (C) 2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -16,26 +16,26 @@
 */
 /**
  * @file lib/exchange_api_melt.c
- * @brief Implementation of the /coins/$COIN_PUB/melt request
- * @author Christian Grothoff
+ * @brief Implementation of the /melt request
+ * @author Özgür Kesim
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <jansson.h>
 #include <microhttpd.h> /* just for HTTP status codes */
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_json_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
-#include "taler_json_lib.h"
-#include "taler_exchange_service.h"
+#include "taler/taler_json_lib.h"
+#include "taler/taler_exchange_service.h"
 #include "exchange_api_common.h"
 #include "exchange_api_handle.h"
-#include "taler_signatures.h"
+#include "taler/taler_signatures.h"
 #include "exchange_api_curl_defaults.h"
 #include "exchange_api_refresh_common.h"
 
 
 /**
- * @brief A /coins/$COIN_PUB/melt Handle
+ * @brief A /melt Handle
  */
 struct TALER_EXCHANGE_MeltHandle
 {
@@ -87,25 +87,37 @@ struct TALER_EXCHANGE_MeltHandle
   struct MeltData md;
 
   /**
-   * The secret the entire melt operation is seeded from.
+   * The seed for the melt operation.
    */
-  struct TALER_RefreshMasterSecretP rms;
+  struct TALER_PublicRefreshMasterSeedP rms;
 
   /**
    * Details about the characteristics of the requested melt operation.
    */
-  const struct TALER_EXCHANGE_RefreshData *rd;
+  const struct TALER_EXCHANGE_MeltInput *rd;
 
   /**
-   * Array of `num_fresh_coins` per-coin values
+   * True, if no blinding_seed is needed (no CS denominations involved)
+   */
+  bool no_blinding_seed;
+
+  /**
+   * If @e no_blinding_seed is false, the blinding seed for the intermediate
+   * call to /blinding-prepare, in order to retrieve the R-values from the
+   * exchange for the blind Clause-Schnorr signature.
+   */
+  struct TALER_BlindingMasterSeedP blinding_seed;
+
+  /**
+   * Array of `num_fresh_denom_pubs` per-coin values
    * returned from melt operation.
    */
-  struct TALER_EXCHANGE_MeltBlindingDetail *mbds;
+  struct TALER_ExchangeBlindingValues *melt_blinding_values;
 
   /**
    * Handle for the preflight request, or NULL.
    */
-  struct TALER_EXCHANGE_CsRMeltHandle *csr;
+  struct TALER_EXCHANGE_BlindingPrepareHandle *bpr;
 
   /**
    * Public key of the coin being melted.
@@ -127,10 +139,6 @@ struct TALER_EXCHANGE_MeltHandle
    */
   uint32_t noreveal_index;
 
-  /**
-   * True if we need to include @e rms in our melt request.
-   */
-  bool send_rms;
 };
 
 
@@ -149,7 +157,6 @@ verify_melt_signature_ok (struct TALER_EXCHANGE_MeltHandle *mh,
                           struct TALER_ExchangePublicKeyP *exchange_pub)
 {
   struct TALER_ExchangeSignatureP exchange_sig;
-  const struct TALER_EXCHANGE_Keys *key_state;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("exchange_sig",
                                  &exchange_sig),
@@ -169,9 +176,8 @@ verify_melt_signature_ok (struct TALER_EXCHANGE_MeltHandle *mh,
     return GNUNET_SYSERR;
   }
   /* check that exchange signing key is permitted */
-  key_state = mh->keys;
   if (GNUNET_OK !=
-      TALER_EXCHANGE_test_signing_key (key_state,
+      TALER_EXCHANGE_test_signing_key (mh->keys,
                                        exchange_pub))
   {
     GNUNET_break_op (0);
@@ -201,7 +207,7 @@ verify_melt_signature_ok (struct TALER_EXCHANGE_MeltHandle *mh,
 
 /**
  * Function called when we're done processing the
- * HTTP /coins/$COIN_PUB/melt request.
+ * HTTP /melt request.
  *
  * @param cls the `struct TALER_EXCHANGE_MeltHandle`
  * @param response_code HTTP response code, 0 on error
@@ -237,8 +243,11 @@ handle_melt_finished (void *cls,
       break;
     }
     mr.details.ok.noreveal_index = mh->noreveal_index;
-    mr.details.ok.num_mbds = mh->rd->fresh_pks_len;
-    mr.details.ok.mbds = mh->mbds;
+    mr.details.ok.num_melt_blinding_values = mh->rd->num_fresh_denom_pubs;
+    mr.details.ok.melt_blinding_values = mh->melt_blinding_values;
+    mr.details.ok.blinding_seed = mh->no_blinding_seed
+                                               ? NULL
+                                               : &mh->blinding_seed;
     mh->melt_cb (mh->melt_cb_cls,
                  &mr);
     mh->melt_cb = NULL;
@@ -300,32 +309,27 @@ handle_melt_finished (void *cls,
 static enum GNUNET_GenericReturnValue
 start_melt (struct TALER_EXCHANGE_MeltHandle *mh)
 {
-  const struct TALER_EXCHANGE_Keys *key_state;
-  json_t *melt_obj;
+  json_t *j_request_body;
+  json_t *j_transfer_pubs;
+  json_t *j_coin_evs;
   CURL *eh;
-  char arg_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2 + 32];
   struct TALER_DenominationHashP h_denom_pub;
-  struct TALER_ExchangeWithdrawValues alg_values[mh->rd->fresh_pks_len];
 
-  for (unsigned int i = 0; i<mh->rd->fresh_pks_len; i++)
-  {
-    if (GNUNET_CRYPTO_BSA_RSA ==
-        mh->rd->fresh_pks[i].key.bsign_pub_key->cipher)
-      alg_values[i] = *TALER_denom_ewv_rsa_singleton ();
-    else
-      alg_values[i] = mh->mbds[i].alg_value;
-  }
   if (GNUNET_OK !=
-      TALER_EXCHANGE_get_melt_data_ (&mh->rms,
-                                     mh->rd,
-                                     alg_values,
-                                     &mh->md))
+      TALER_EXCHANGE_get_melt_data (&mh->rms,
+                                    mh->rd,
+                                    mh->no_blinding_seed
+                                        ? NULL
+                                        : &mh->blinding_seed,
+                                    mh->melt_blinding_values,
+                                    &mh->md))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  TALER_denom_pub_hash (&mh->md.melted_coin.pub_key,
-                        &h_denom_pub);
+  TALER_denom_pub_hash (
+    &mh->md.melted_coin.pub_key,
+    &h_denom_pub);
   TALER_wallet_melt_sign (
     &mh->md.melted_coin.melt_amount_with_fee,
     &mh->md.melted_coin.fee_melt,
@@ -334,59 +338,91 @@ start_melt (struct TALER_EXCHANGE_MeltHandle *mh)
     mh->md.melted_coin.h_age_commitment,
     &mh->md.melted_coin.coin_priv,
     &mh->coin_sig);
-  GNUNET_CRYPTO_eddsa_key_get_public (&mh->md.melted_coin.coin_priv.eddsa_priv,
-                                      &mh->coin_pub.eddsa_pub);
-  melt_obj = GNUNET_JSON_PACK (
-    GNUNET_JSON_pack_data_auto ("denom_pub_hash",
+  GNUNET_CRYPTO_eddsa_key_get_public (
+    &mh->md.melted_coin.coin_priv.eddsa_priv,
+    &mh->coin_pub.eddsa_pub);
+  mh->dki = TALER_EXCHANGE_get_denomination_key (
+    mh->keys,
+    &mh->md.melted_coin.pub_key);
+  j_request_body = GNUNET_JSON_PACK (
+    GNUNET_JSON_pack_data_auto ("old_coin_pub",
+                                &mh->coin_pub),
+    GNUNET_JSON_pack_data_auto ("old_denom_pub_h",
                                 &h_denom_pub),
-    TALER_JSON_pack_denom_sig ("denom_sig",
+    TALER_JSON_pack_denom_sig ("old_denom_sig",
                                &mh->md.melted_coin.sig),
     GNUNET_JSON_pack_data_auto ("confirm_sig",
                                 &mh->coin_sig),
     TALER_JSON_pack_amount ("value_with_fee",
                             &mh->md.melted_coin.melt_amount_with_fee),
-    GNUNET_JSON_pack_data_auto ("rc",
-                                &mh->md.rc),
     GNUNET_JSON_pack_allow_null (
       (NULL != mh->md.melted_coin.h_age_commitment)
-      ? GNUNET_JSON_pack_data_auto ("age_commitment_hash",
+      ? GNUNET_JSON_pack_data_auto ("old_age_commitment_h",
                                     mh->md.melted_coin.h_age_commitment)
-      : GNUNET_JSON_pack_string ("age_commitment_hash",
+      : GNUNET_JSON_pack_string ("old_age_commitment_h",
                                  NULL)),
+    GNUNET_JSON_pack_data_auto ("refresh_seed",
+                                &mh->md.refresh_seed),
     GNUNET_JSON_pack_allow_null (
-      mh->send_rms
-      ? GNUNET_JSON_pack_data_auto ("rms",
-                                    &mh->rms)
-      : GNUNET_JSON_pack_string ("rms",
-                                 NULL)));
+      (mh->md.no_blinding_seed)
+      ? GNUNET_JSON_pack_string ("blinding_seed",
+                                 NULL)
+      : GNUNET_JSON_pack_data_auto ("blinding_seed",
+                                    &mh->md.blinding_seed)),
+    TALER_JSON_pack_array_of_data_auto ("denoms_h",
+                                        mh->md.num_fresh_coins,
+                                        mh->md.denoms_h)
+    );
+  GNUNET_assert (NULL != j_request_body);
+  GNUNET_assert (NULL !=
+                 (j_transfer_pubs = json_array ()));
+  GNUNET_assert (NULL !=
+                 (j_coin_evs = json_array ()));
+  /**
+   * Fill the kappa array of coin envelopes and
+   * the array of transfer pubs.
+   */
+  for (uint8_t k=0; k<TALER_CNC_KAPPA; k++)
   {
-    char pub_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2];
-    char *end;
+    json_t *j_envs;
+    json_t *j_tbs = GNUNET_JSON_PACK (
+      TALER_JSON_pack_array_of_data_auto (NULL,
+                                          mh->md.num_fresh_coins,
+                                          mh->md.kappa_transfer_pubs[k])
+      );
 
-    end = GNUNET_STRINGS_data_to_string (
-      &mh->coin_pub,
-      sizeof (struct TALER_CoinSpendPublicKeyP),
-      pub_str,
-      sizeof (pub_str));
-    *end = '\0';
-    GNUNET_snprintf (arg_str,
-                     sizeof (arg_str),
-                     "coins/%s/melt",
-                     pub_str);
+    GNUNET_assert (NULL != (j_envs = json_array ()));
+    GNUNET_assert (NULL !=j_tbs);
+
+    for (size_t i = 0; i < mh->md.num_fresh_coins; i++)
+    {
+      json_t *j_coin = GNUNET_JSON_PACK (
+        TALER_JSON_pack_blinded_planchet (NULL,
+                                          &mh->md.kappa_blinded_planchets[k][i]));
+      GNUNET_assert (NULL != j_coin);
+      GNUNET_assert (0 ==
+                     json_array_append_new (j_envs, j_coin));
+    }
+    GNUNET_assert (0 ==
+                   json_array_append_new (j_coin_evs, j_envs));
+    GNUNET_assert (0 ==
+                   json_array_append_new (j_transfer_pubs, j_tbs));
   }
-
-  key_state = mh->keys;
-  mh->dki = TALER_EXCHANGE_get_denomination_key (key_state,
-                                                 &mh->md.melted_coin.pub_key);
-
+  GNUNET_assert (0 ==
+                 json_object_set_new (j_request_body,
+                                      "coin_evs",
+                                      j_coin_evs));
+  GNUNET_assert (0 ==
+                 json_object_set_new (j_request_body,
+                                      "transfer_pubs",
+                                      j_transfer_pubs));
   /* and now we can at last begin the actual request handling */
-
   mh->url = TALER_url_join (mh->exchange_url,
-                            arg_str,
+                            "melt",
                             NULL);
   if (NULL == mh->url)
   {
-    json_decref (melt_obj);
+    json_decref (j_request_body);
     return GNUNET_SYSERR;
   }
   eh = TALER_EXCHANGE_curl_easy_get_ (mh->url);
@@ -394,15 +430,15 @@ start_melt (struct TALER_EXCHANGE_MeltHandle *mh)
        (GNUNET_OK !=
         TALER_curl_easy_post (&mh->ctx,
                               eh,
-                              melt_obj)) )
+                              j_request_body)) )
   {
     GNUNET_break (0);
     if (NULL != eh)
       curl_easy_cleanup (eh);
-    json_decref (melt_obj);
+    json_decref (j_request_body);
     return GNUNET_SYSERR;
   }
-  json_decref (melt_obj);
+  json_decref (j_request_body);
   mh->job = GNUNET_CURL_job_add2 (mh->cctx,
                                   eh,
                                   mh->ctx.headers,
@@ -435,36 +471,36 @@ fail_mh (struct TALER_EXCHANGE_MeltHandle *mh,
 
 /**
  * Callbacks of this type are used to serve the result of submitting a
- * CS R request to a exchange.
+ * /blinding-prepare request to a exchange.
  *
  * @param cls closure with our `struct TALER_EXCHANGE_MeltHandle *`
- * @param csrr response details
+ * @param bpr response details
  */
 static void
-csr_cb (void *cls,
-        const struct TALER_EXCHANGE_CsRMeltResponse *csrr)
+blinding_prepare_cb (void *cls,
+                     const struct TALER_EXCHANGE_BlindingPrepareResponse *bpr)
 {
   struct TALER_EXCHANGE_MeltHandle *mh = cls;
   unsigned int nks_off = 0;
 
-  mh->csr = NULL;
-  if (MHD_HTTP_OK != csrr->hr.http_status)
+  mh->bpr = NULL;
+  if (MHD_HTTP_OK != bpr->hr.http_status)
   {
     struct TALER_EXCHANGE_MeltResponse mr = {
-      .hr = csrr->hr
+      .hr = bpr->hr
     };
 
-    mr.hr.hint = "/csr-melt failed";
+    mr.hr.hint = "/blinding-prepare failed";
     mh->melt_cb (mh->melt_cb_cls,
                  &mr);
     TALER_EXCHANGE_melt_cancel (mh);
     return;
   }
-  for (unsigned int i = 0; i<mh->rd->fresh_pks_len; i++)
+  for (unsigned int i = 0; i<mh->rd->num_fresh_denom_pubs; i++)
   {
     const struct TALER_EXCHANGE_DenomPublicKey *fresh_pk =
-      &mh->rd->fresh_pks[i];
-    struct TALER_ExchangeWithdrawValues *wv = &mh->mbds[i].alg_value;
+      &mh->rd->fresh_denom_pubs[i];
+    struct TALER_ExchangeBlindingValues *wv = &mh->melt_blinding_values[i];
 
     switch (fresh_pk->key.bsign_pub_key->cipher)
     {
@@ -477,12 +513,11 @@ csr_cb (void *cls,
       break;
     case GNUNET_CRYPTO_BSA_CS:
       TALER_denom_ewv_copy (wv,
-                            &csrr->details.ok.alg_values[nks_off]);
+                            &bpr->details.ok.blinding_values[nks_off]);
       nks_off++;
       break;
     }
   }
-  mh->send_rms = true;
   if (GNUNET_OK !=
       start_melt (mh))
   {
@@ -499,16 +534,16 @@ TALER_EXCHANGE_melt (
   struct GNUNET_CURL_Context *ctx,
   const char *url,
   struct TALER_EXCHANGE_Keys *keys,
-  const struct TALER_RefreshMasterSecretP *rms,
-  const struct TALER_EXCHANGE_RefreshData *rd,
+  const struct TALER_PublicRefreshMasterSeedP *rms,
+  const struct TALER_EXCHANGE_MeltInput *rd,
   TALER_EXCHANGE_MeltCallback melt_cb,
   void *melt_cb_cls)
 {
-  struct TALER_EXCHANGE_NonceKey nks[GNUNET_NZL (rd->fresh_pks_len)];
+  struct TALER_EXCHANGE_NonceKey nks[GNUNET_NZL (rd->num_fresh_denom_pubs)];
   unsigned int nks_off = 0;
   struct TALER_EXCHANGE_MeltHandle *mh;
 
-  if (0 == rd->fresh_pks_len)
+  if (0 == rd->num_fresh_denom_pubs)
   {
     GNUNET_break (0);
     return NULL;
@@ -521,26 +556,29 @@ TALER_EXCHANGE_melt (
   mh->rms = *rms;
   mh->melt_cb = melt_cb;
   mh->melt_cb_cls = melt_cb_cls;
-  mh->mbds = GNUNET_new_array (rd->fresh_pks_len,
-                               struct TALER_EXCHANGE_MeltBlindingDetail);
-  for (unsigned int i = 0; i<rd->fresh_pks_len; i++)
+  mh->no_blinding_seed = true;
+  mh->melt_blinding_values =
+    GNUNET_new_array (rd->num_fresh_denom_pubs,
+                      struct TALER_ExchangeBlindingValues);
+  for (unsigned int i = 0; i<rd->num_fresh_denom_pubs; i++)
   {
-    const struct TALER_EXCHANGE_DenomPublicKey *fresh_pk = &rd->fresh_pks[i];
+    const struct TALER_EXCHANGE_DenomPublicKey *fresh_pk =
+      &rd->fresh_denom_pubs[i];
 
     switch (fresh_pk->key.bsign_pub_key->cipher)
     {
     case GNUNET_CRYPTO_BSA_INVALID:
       GNUNET_break (0);
-      GNUNET_free (mh->mbds);
+      GNUNET_free (mh->melt_blinding_values);
       GNUNET_free (mh);
       return NULL;
     case GNUNET_CRYPTO_BSA_RSA:
-      TALER_denom_ewv_copy (&mh->mbds[i].alg_value,
+      TALER_denom_ewv_copy (&mh->melt_blinding_values[i],
                             TALER_denom_ewv_rsa_singleton ());
       break;
     case GNUNET_CRYPTO_BSA_CS:
       nks[nks_off].pk = fresh_pk;
-      nks[nks_off].cnc_num = nks_off;
+      nks[nks_off].cnc_num = i;
       nks_off++;
       break;
     }
@@ -548,14 +586,19 @@ TALER_EXCHANGE_melt (
   mh->keys = TALER_EXCHANGE_keys_incref (keys);
   if (0 != nks_off)
   {
-    mh->csr = TALER_EXCHANGE_csr_melt (ctx,
-                                       url,
-                                       rms,
-                                       nks_off,
-                                       nks,
-                                       &csr_cb,
-                                       mh);
-    if (NULL == mh->csr)
+    mh->no_blinding_seed = false;
+    TALER_cs_refresh_seed_to_blinding_seed (
+      rms,
+      &mh->md.melted_coin.coin_priv,
+      &mh->blinding_seed);
+    mh->bpr = TALER_EXCHANGE_blinding_prepare_for_melt (ctx,
+                                                        url,
+                                                        &mh->blinding_seed,
+                                                        nks_off,
+                                                        nks,
+                                                        &blinding_prepare_cb,
+                                                        mh);
+    if (NULL == mh->bpr)
     {
       GNUNET_break (0);
       TALER_EXCHANGE_melt_cancel (mh);
@@ -577,20 +620,20 @@ TALER_EXCHANGE_melt (
 void
 TALER_EXCHANGE_melt_cancel (struct TALER_EXCHANGE_MeltHandle *mh)
 {
-  for (unsigned int i = 0; i<mh->rd->fresh_pks_len; i++)
-    TALER_denom_ewv_free (&mh->mbds[i].alg_value);
+  for (unsigned int i = 0; i<mh->rd->num_fresh_denom_pubs; i++)
+    TALER_denom_ewv_free (&mh->melt_blinding_values[i]);
   if (NULL != mh->job)
   {
     GNUNET_CURL_job_cancel (mh->job);
     mh->job = NULL;
   }
-  if (NULL != mh->csr)
+  if (NULL != mh->bpr)
   {
-    TALER_EXCHANGE_csr_melt_cancel (mh->csr);
-    mh->csr = NULL;
+    TALER_EXCHANGE_blinding_prepare_cancel (mh->bpr);
+    mh->bpr = NULL;
   }
-  TALER_EXCHANGE_free_melt_data_ (&mh->md); /* does not free 'md' itself */
-  GNUNET_free (mh->mbds);
+  TALER_EXCHANGE_free_melt_data (&mh->md); /* does not free 'md' itself */
+  GNUNET_free (mh->melt_blinding_values);
   GNUNET_free (mh->url);
   GNUNET_free (mh->exchange_url);
   TALER_curl_easy_post_finished (&mh->ctx);
