@@ -215,6 +215,7 @@ handle_recoup_refresh_finished (void *cls,
 }
 
 
+#pragma message "TODO[oec]: rewrite TALER_EXCHANGE_RecoupRefreshHandle"
 struct TALER_EXCHANGE_RecoupRefreshHandle *
 TALER_EXCHANGE_recoup_refresh (
   struct GNUNET_CURL_Context *ctx,
@@ -223,7 +224,7 @@ TALER_EXCHANGE_recoup_refresh (
   const struct TALER_EXCHANGE_DenomPublicKey *pk,
   const struct TALER_DenominationSignature *denom_sig,
   const struct TALER_ExchangeBlindingValues *blinding_values,
-  const struct TALER_RefreshMasterSecretP *rms,
+  const struct TALER_PublicRefreshMasterSeedP *rms,
   const struct TALER_PlanchetMasterSecretP *ps,
   unsigned int idx,
   TALER_EXCHANGE_RecoupRefreshResultCallback recoup_cb,
@@ -282,21 +283,7 @@ TALER_EXCHANGE_recoup_refresh (
     break;
   case GNUNET_CRYPTO_BSA_CS:
     {
-      union GNUNET_CRYPTO_BlindSessionNonce nonce;
 
-      /* NOTE: this is not elegant, and as per the note in TALER_coin_ev_hash()
-         it is not strictly clear that the nonce is needed. Best case would be
-         to find a way to include it more 'naturally' somehow, for example with
-         the variant union version of bks! */
-      TALER_cs_refresh_nonce_derive (rms,
-                                     idx,
-                                     &nonce.cs_nonce);
-      GNUNET_assert (
-        0 ==
-        json_object_set_new (recoup_obj,
-                             "nonce",
-                             GNUNET_JSON_from_data_auto (
-                               &nonce)));
     }
     break;
   }
