@@ -20,18 +20,19 @@
  * @author Christian Grothoff
  * @author Özgür Kesim
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include "exchange_api_refresh_common.h"
 
 
 void
-TALER_EXCHANGE_free_melt_data_v27 (struct MeltData_v27 *md)
+TALER_EXCHANGE_free_melt_data (struct MeltData *md)
 {
   for (unsigned int k = 0; k < TALER_CNC_KAPPA; k++)
   {
     for (unsigned int i = 0; i < md->num_fresh_coins; i++)
       TALER_blinded_planchet_free (&md->kappa_blinded_planchets[k][i]);
     GNUNET_free (md->kappa_blinded_planchets[k]);
+    GNUNET_free (md->kappa_transfer_pubs[k]);
   }
   GNUNET_free (md->denoms_h);
   GNUNET_free (md->denom_pubs);
@@ -54,23 +55,22 @@ TALER_EXCHANGE_free_melt_data_v27 (struct MeltData_v27 *md)
   }
   /* Finally, clean up a bit... */
   GNUNET_CRYPTO_zero_keys (md,
-                           sizeof (struct MeltData_v27));
+                           sizeof (struct MeltData));
 }
 
 
 enum GNUNET_GenericReturnValue
-TALER_EXCHANGE_get_melt_data_v27 (
-  const struct TALER_RefreshMasterSecretP *rms,
+TALER_EXCHANGE_get_melt_data (
+  const struct TALER_PublicRefreshMasterSeedP *rms,
   const struct TALER_EXCHANGE_MeltInput *rd,
   const struct TALER_BlindingMasterSeedP *blinding_seed,
   const struct TALER_ExchangeBlindingValues *blinding_values,
-  struct MeltData_v27 *md)
+  struct MeltData *md)
 {
   struct TALER_Amount total;
   struct TALER_CoinSpendPublicKeyP coin_pub;
   struct TALER_KappaHashBlindedPlanchetsP k_h_bps;
   union GNUNET_CRYPTO_BlindSessionNonce nonces[rd->num_fresh_denom_pubs];
-  const struct TALER_DenominationHashP *denoms_h[rd->num_fresh_denom_pubs];
   bool is_cs[rd->num_fresh_denom_pubs];
   bool uses_cs = false;
 
@@ -80,6 +80,7 @@ TALER_EXCHANGE_get_melt_data_v27 (
           0,
           sizeof (*md));
   /* build up melt data structure */
+  md->refresh_seed = *rms;
   md->num_fresh_coins = rd->num_fresh_denom_pubs;
   md->melted_coin.coin_priv = rd->melt_priv;
   md->melted_coin.melt_amount_with_fee = rd->melt_amount;
@@ -108,11 +109,10 @@ TALER_EXCHANGE_get_melt_data_v27 (
     GNUNET_new_array (md->num_fresh_coins,
                       const struct  TALER_DenominationPublicKey*);
 
-  for (unsigned int j = 0; j<rd->num_fresh_denom_pubs; j++)
+  for (unsigned int j = 0; j<md->num_fresh_coins; j++)
   {
     struct FreshCoinData *fcd = &md->fcds[j];
 
-    denoms_h[j] = &rd->fresh_denom_pubs[j].h_key;
     md->denoms_h[j] = rd->fresh_denom_pubs[j].h_key;
     md->denom_pubs[j] = &rd->fresh_denom_pubs[j].key;
 
@@ -123,14 +123,14 @@ TALER_EXCHANGE_get_melt_data_v27 (
         fcd->fresh_pk.bsign_pub_key->cipher)
     {
       GNUNET_break (0);
-      TALER_EXCHANGE_free_melt_data_v27 (md);
+      TALER_EXCHANGE_free_melt_data (md);
       return GNUNET_SYSERR;
     }
     switch (fcd->fresh_pk.bsign_pub_key->cipher)
     {
     case GNUNET_CRYPTO_BSA_INVALID:
       GNUNET_break (0);
-      TALER_EXCHANGE_free_melt_data_v27 (md);
+      TALER_EXCHANGE_free_melt_data (md);
       return GNUNET_SYSERR;
     case GNUNET_CRYPTO_BSA_RSA:
       is_cs[j] = false;
@@ -150,7 +150,7 @@ TALER_EXCHANGE_get_melt_data_v27 (
                             &rd->fresh_denom_pubs[j].fees.withdraw)) )
     {
       GNUNET_break (0);
-      TALER_EXCHANGE_free_melt_data_v27 (md);
+      TALER_EXCHANGE_free_melt_data (md);
       return GNUNET_SYSERR;
     }
   }
@@ -163,13 +163,13 @@ TALER_EXCHANGE_get_melt_data_v27 (
     /* Eh, this operation is more expensive than the
        @a melt_amount. This is not OK. */
     GNUNET_break (0);
-    TALER_EXCHANGE_free_melt_data_v27 (md);
+    TALER_EXCHANGE_free_melt_data (md);
     return GNUNET_SYSERR;
   }
   /**
    * Generate the blinding seeds and nonces for CS.
    * Note that blinding_seed was prepared upstream,
-   * in TALER_EXCHANGE_melt_v27(), as preparation for
+   * in TALER_EXCHANGE_melt(), as preparation for
    * the request to `/blinding-prepare`.
    */
   memset (nonces,
@@ -181,53 +181,48 @@ TALER_EXCHANGE_get_melt_data_v27 (
     TALER_cs_derive_blind_nonces_from_seed (
       blinding_seed,
       true, /* for melt */
-      rd->num_fresh_denom_pubs,
+      md->num_fresh_coins,
       is_cs,
       nonces);
   }
   /**
-   * Generate from the master secret the refresh seed
-   * and kappa nonces
+   * Generate the kappa private seeds for the batches.
    */
-  TALER_refresh_master_secret_to_refresh_seed (
-    rms,
-    &md->refresh_seed);
-  TALER_refresh_expand_kappa_nonces (
+  TALER_refresh_expand_seed_to_kappa_batch_seeds (
     &md->refresh_seed,
-    &md->kappa_nonces);
+    &rd->melt_priv,
+    &md->kappa_batch_seeds);
   /**
    * Build up all candidates for coin planchets
    */
   for (unsigned int k = 0; k<TALER_CNC_KAPPA; k++)
   {
     struct TALER_PlanchetMasterSecretP
-      planchet_secrets[rd->num_fresh_denom_pubs];
-    struct TALER_PlanchetDetail planchet_details[rd->num_fresh_denom_pubs];
-
-    TALER_wallet_refresh_nonce_sign (
-      &rd->melt_priv,
-      &md->kappa_nonces.tuple[k],
-      rd->num_fresh_denom_pubs,
-      denoms_h,
-      k,
-      &md->signatures[k]);
-    TALER_refresh_signature_to_secrets (
-      &md->signatures[k],
-      rd->num_fresh_denom_pubs,
-      planchet_secrets);
+      planchet_secrets[md->num_fresh_coins];
+    struct TALER_PlanchetDetail planchet_details[md->num_fresh_coins];
 
     md->kappa_blinded_planchets[k] =
-      GNUNET_new_array (rd->num_fresh_denom_pubs,
-                        struct  TALER_BlindedPlanchet);
+      GNUNET_new_array (md->num_fresh_coins,
+                        struct TALER_BlindedPlanchet);
+    md->kappa_transfer_pubs[k] =
+      GNUNET_new_array (md->num_fresh_coins,
+                        struct TALER_TransferPublicKeyP);
 
-    for (unsigned int j = 0; j<rd->num_fresh_denom_pubs; j++)
+    TALER_refresh_expand_batch_seed_to_transfer_data (
+      &md->kappa_batch_seeds.tuple[k],
+      &coin_pub,
+      md->num_fresh_coins,
+      planchet_secrets,
+      md->kappa_transfer_pubs[k]);
+
+    for (unsigned int j = 0; j<md->num_fresh_coins; j++)
     {
       struct FreshCoinData *fcd = &md->fcds[j];
       struct TALER_CoinSpendPrivateKeyP *coin_priv = &fcd->coin_priv;
       union GNUNET_CRYPTO_BlindingSecretP *bks = &fcd->bks[k];
       struct TALER_CoinPubHashP c_hash;
-      struct TALER_AgeCommitmentHash ach;
-      struct TALER_AgeCommitmentHash *pah;
+      struct TALER_AgeCommitmentHashP ach;
+      struct TALER_AgeCommitmentHashP *pah;
 
       fcd->ps[k] = planchet_secrets[j];
       TALER_planchet_setup_coin_priv (&planchet_secrets[j],
@@ -238,8 +233,8 @@ TALER_EXCHANGE_get_melt_data_v27 (
                                              bks);
       if (NULL != rd->melt_age_commitment_proof)
       {
-        fcd->age_commitment_proofs[k] = GNUNET_new (struct
-                                                    TALER_AgeCommitmentProof);
+        fcd->age_commitment_proofs[k] =
+          GNUNET_new (struct TALER_AgeCommitmentProof);
         GNUNET_assert (GNUNET_OK ==
                        TALER_age_commitment_proof_derive_from_secret (
                          md->melted_coin.age_commitment_proof,
@@ -266,7 +261,7 @@ TALER_EXCHANGE_get_melt_data_v27 (
                                   &planchet_details[j]))
       {
         GNUNET_break_op (0);
-        TALER_EXCHANGE_free_melt_data_v27 (md);
+        TALER_EXCHANGE_free_melt_data (md);
         return GNUNET_SYSERR;
       }
       md->kappa_blinded_planchets[k][j] =
@@ -282,13 +277,23 @@ TALER_EXCHANGE_get_melt_data_v27 (
   }
 
   /* Finally, compute refresh commitment */
-  TALER_refresh_get_commitment_v27 (&md->rc,
-                                    &md->refresh_seed,
-                                    uses_cs
+  {
+    struct TALER_KappaTransferPublicKeys k_tr_pubs = {
+      .num_transfer_pubs = md->num_fresh_coins,
+    };
+
+    for (uint8_t k=0; k<TALER_CNC_KAPPA; k++)
+      k_tr_pubs.batch[k] = md->kappa_transfer_pubs[k];
+
+    TALER_refresh_get_commitment (&md->rc,
+                                  &md->refresh_seed,
+                                  uses_cs
                                          ? &md->blinding_seed
                                          : NULL,
-                                    &k_h_bps,
-                                    &coin_pub,
-                                    &rd->melt_amount);
+                                  &k_tr_pubs,
+                                  &k_h_bps,
+                                  &coin_pub,
+                                  &rd->melt_amount);
+  }
   return GNUNET_OK;
 }
