@@ -117,7 +117,7 @@ struct FreshCoinData
 /**
  * Melt data in non-serialized format for convenient processing.
  */
-struct MeltData_v27
+struct MeltData
 {
 
   /**
@@ -153,17 +153,10 @@ struct MeltData_v27
   struct TALER_PublicRefreshMasterSeedP refresh_seed;
 
   /**
-   * The tuple of kappa nonces which we sign with the melted coin.
-   * From those signatures, all the planchet seeds are derived from.
+   * The tuple of kappa batch seeds, derived from the @a refresh_seed
+   * and the coin's private key.
    */
-  struct TALER_KappaPublicRefreshNoncesP kappa_nonces;
-
-  /**
-   * The tuple of kappa signatures of the @e kappa_nonces, signed
-   * by the @e melted_coin, from which we derive all
-   * kappa * @e num_fresh_coins planchet secrets.
-   */
-  struct TALER_PrivateRefreshNonceSignatureP signatures[TALER_CNC_KAPPA];
+  struct TALER_KappaPrivateRefreshBatchSeedsP kappa_batch_seeds;
 
   /**
    * Array of length @e num_fresh_coins with information
@@ -181,6 +174,13 @@ struct MeltData_v27
    */
   struct TALER_DenominationHashP  *denoms_h;
 
+  /**
+   * kappa many arrays fo @e num_fresh_coins transfer public keys that are
+   * derived from the @e kappa_batch_seeds and the private key of the coin.
+   * This will be sent to the exchange during the melt request.
+   */
+  struct TALER_TransferPublicKeyP *kappa_transfer_pubs[TALER_CNC_KAPPA];
+
   /**
    * Blinded planchets of the fresh coins, depending on the cut-and-choose.
    * Array of length @e num_fresh_coins.
@@ -192,19 +192,19 @@ struct MeltData_v27
  * Compute the melt data from the refresh data and secret
  * for v27 of the protocol.
  *
- * @param rms secret internals of the refresh-reveal operation
+ * @param rms public seed of the refresh-reveal operation
  * @param rd refresh data with the characteristics of the operation
  * @param blinding_seed seed for blinding in case of CS denominations, can be NULL
  * @param alg_values contributions from the exchange into the melt
  * @param[out] md where to write the derived melt data
  */
 enum GNUNET_GenericReturnValue
-TALER_EXCHANGE_get_melt_data_v27 (
-  const struct TALER_RefreshMasterSecretP *rms,
+TALER_EXCHANGE_get_melt_data (
+  const struct TALER_PublicRefreshMasterSeedP *rms,
   const struct TALER_EXCHANGE_MeltInput *rd,
   const struct TALER_BlindingMasterSeedP *blinding_seed,
   const struct TALER_ExchangeBlindingValues *alg_values,
-  struct MeltData_v27 *md);
+  struct MeltData *md);
 
 /**
  * Free all information associated with a melting session.  Note
@@ -216,6 +216,6 @@ TALER_EXCHANGE_get_melt_data_v27 (
  *           freed (as it is typically not allocated by itself)
  */
 void
-TALER_EXCHANGE_free_melt_data_v27 (struct MeltData_v27 *md);
+TALER_EXCHANGE_free_melt_data (struct MeltData *md);
 
 #endif
