@@ -67,7 +67,7 @@ struct TALER_EXCHANGE_RevealMeltHandle
   /**
    * The melt data
    */
-  struct MeltData_v27 md;
+  struct MeltData md;
 
   /**
    * Callback to pass the result onto
@@ -296,11 +296,11 @@ perform_protocol (
   struct TALER_EXCHANGE_RevealMeltHandle *mrh)
 {
   CURL *curlh;
-  json_t *j_array_of_signatures;
+  json_t *j_batch_seeds;
 
 
-  j_array_of_signatures = json_array ();
-  GNUNET_assert (NULL != j_array_of_signatures);
+  j_batch_seeds = json_array ();
+  GNUNET_assert (NULL != j_batch_seeds);
 
   for (uint8_t k = 0; k < TALER_CNC_KAPPA; k++)
   {
@@ -308,8 +308,9 @@ perform_protocol (
       continue;
 
     GNUNET_assert (0 == json_array_append_new (
-                     j_array_of_signatures,
-                     GNUNET_JSON_from_data_auto (&mrh->md.signatures[k])));
+                     j_batch_seeds,
+                     GNUNET_JSON_from_data_auto (
+                       &mrh->md.kappa_batch_seeds.tuple[k])));
   }
   {
     json_t *j_request_body;
@@ -317,8 +318,8 @@ perform_protocol (
     j_request_body = GNUNET_JSON_PACK (
       GNUNET_JSON_pack_data_auto ("rc",
                                   &mrh->md.rc),
-      GNUNET_JSON_pack_array_steal ("signatures",
-                                    j_array_of_signatures));
+      GNUNET_JSON_pack_array_steal ("batch_seeds",
+                                    j_batch_seeds));
     GNUNET_assert (NULL != j_request_body);
 
     if (NULL != mrh->reveal_input->melt_input->melt_age_commitment_proof)
@@ -341,7 +342,6 @@ perform_protocol (
                                          j_request_body));
     json_decref (j_request_body);
   }
-
   mrh->job = GNUNET_CURL_job_add2 (
     curl_ctx,
     curlh,
@@ -388,7 +388,7 @@ TALER_EXCHANGE_reveal_melt (
     GNUNET_free (mrh);
     return NULL;
   }
-  TALER_EXCHANGE_get_melt_data_v27 (
+  TALER_EXCHANGE_get_melt_data (
     reveal_melt_input->rms,
     reveal_melt_input->melt_input,
     reveal_melt_input->blinding_seed,
@@ -410,7 +410,7 @@ TALER_EXCHANGE_reveal_melt_cancel (
     mrh->job = NULL;
   }
   TALER_curl_easy_post_finished (&mrh->post_ctx);
-  TALER_EXCHANGE_free_melt_data_v27 (&mrh->md);
+  TALER_EXCHANGE_free_melt_data (&mrh->md);
   GNUNET_free (mrh->request_url);
   GNUNET_free (mrh);
 }
