@@ -70,18 +70,20 @@ TALER_MHD2_add_global_headers (struct MHD_Response *response,
 bool
 TALER_MHD2_can_compress (struct MHD_Request *request)
 {
-  const struct MHD_StringNullable *aeb;
+  struct MHD_StringNullable aeb;
   const char *ae;
   const char *de;
 
   if (0 != (TM_go & TALER_MHD2_GO_DISABLE_COMPRESSION))
     return MHD_NO;
-  aeb = MHD_request_get_value (request,
+  if ( (MHD_NO ==
+        MHD_request_get_value (request,
                                MHD_VK_HEADER,
-                               MHD_HTTP_HEADER_ACCEPT_ENCODING);
-  if (NULL == aeb)
+                               MHD_HTTP_HEADER_ACCEPT_ENCODING,
+                               &aeb)) ||
+       (NULL == aeb.cstr) )
     return false;
-  ae = aeb->cstr;
+  ae = aeb.cstr;
   if (NULL == ae)
     return false;
   if (0 == strcmp (ae,
