@@ -370,7 +370,7 @@ complete_response (void *cls)
     NULL,
     NULL,
     NULL,
-    "pdftk",
+    argv[0],
     (char **) argv);
   if (NULL == tc->proc)
   {
@@ -550,9 +550,7 @@ setup_stage (struct TypstStage *stage,
     if (0 !=
         json_dump_file (doc->data,
                         jfn,
-                        JSON_INDENT (2)
-                        // JSON_COMPACT
-                        ))
+                        JSON_INDENT (2)))
     {
       GNUNET_break (0);
       GNUNET_free (jfn);
