@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2023 Taler Systems SA
+  Copyright (C) 2014-2026 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -17,6 +17,7 @@
  * @file pq/pq_result_helper.c
  * @brief functions to initialize parameter arrays
  * @author Christian Grothoff
+ * @author Özgür Kesim
  */
 #include "taler/platform.h"
 #include <gnunet/gnunet_util_lib.h>
@@ -1163,13 +1164,13 @@ extract_array_generic (
   *((void **) dst) = NULL;
 
   #define FAIL_IF(cond) \
-  do { \
-    if ((cond)) \
-    { \
-      GNUNET_break (! (cond)); \
-      goto FAIL; \
-    } \
-  } while (0)
+          do { \
+            if ((cond)) \
+            { \
+              GNUNET_break (! (cond)); \
+              goto FAIL; \
+            } \
+          } while (0)
 
   col_num = PQfnumber (result, fname);
   FAIL_IF (0 > col_num);
@@ -1181,9 +1182,28 @@ extract_array_generic (
 
   data_sz = PQgetlength (result, row, col_num);
   FAIL_IF (0 > data_sz);
-  FAIL_IF (sizeof(header) > (size_t) data_sz);
+
+  /* Report if this field is empty */
+  if (0 == (size_t) data_sz)
+    return GNUNET_NO;
 
   data = PQgetvalue (result, row, col_num);
+
+  if (sizeof(header) > (size_t) data_sz)
+  {
+    uint32_t ndim;
+
+    /* data_sz is shorter than header if the
+       array length is 0, in which case ndim is 0! */
+    FAIL_IF (sizeof(uint32_t) > (size_t) data_sz);
+    memcpy (&ndim,
+            data,
+            sizeof (ndim));
+    FAIL_IF (0 != ndim);
+    *info->num = 0;
+    return GNUNET_OK;
+  }
+  FAIL_IF (sizeof(header) > (size_t) data_sz);
   FAIL_IF (NULL == data);
 
   {
