@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2015, 2016, 2023 Taler Systems SA
+  (C) 2015, 2016, 2023, 2026 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -17,6 +17,7 @@
  * @file pq/test_pq.c
  * @brief Tests for Postgres convenience API
  * @author Christian Grothoff <christian@grothoff.org>
+ * @author Özgür Kesim <oec-taler@kesim.org>
  */
 #include "taler/platform.h"
 #include "taler/taler_util.h"
@@ -36,7 +37,7 @@ postgres_prepare (struct GNUNET_PQ_Context *db)
   struct GNUNET_PQ_PreparedStatement ps[] = {
     GNUNET_PQ_make_prepare ("test_insert",
                             "INSERT INTO test_pq ("
-                            " tamount"
+                            "tamount"
                             ",json"
                             ",aamount"
                             ",aamountc"
@@ -52,6 +53,8 @@ postgres_prepare (struct GNUNET_PQ_Context *db)
                             ",json"
                             ",aamount"
                             ",aamountc"
+                            ",aamountn"
+                            ",aamountnc"
                             ",tamountc"
                             ",hash"
                             ",hashes"
@@ -177,11 +180,15 @@ run_queries (struct GNUNET_PQ_Context *conn)
     struct TALER_Amount tamountc2;
     struct TALER_Amount *pamount;
     struct TALER_Amount *pamountc;
+    struct TALER_Amount *pamountn;
+    struct TALER_Amount *pamountnc;
     struct GNUNET_HashCode hc2;
     struct GNUNET_HashCode *hcs2;
     struct GNUNET_CRYPTO_CSPublicRPairP *out_cs_r_pubs;
     size_t npamount;
     size_t npamountc;
+    size_t npamountn;
+    size_t npamountnc;
     size_t nhcs;
     size_t n_rpubs;
     json_t *json2;
@@ -203,6 +210,15 @@ run_queries (struct GNUNET_PQ_Context *conn)
                                                        "aamountc",
                                                        &npamountc,
                                                        &pamountc),
+      TALER_PQ_result_spec_array_amount (conn,
+                                         "aamountn",
+                                         "EUR",
+                                         &npamountn,
+                                         &pamountn),
+      TALER_PQ_result_spec_array_amount_with_currency (conn,
+                                                       "aamountnc",
+                                                       &npamountnc,
+                                                       &pamountnc),
       TALER_PQ_result_spec_amount_with_currency ("tamountc",
                                                  &tamountc2),
       GNUNET_PQ_result_spec_auto_from_type ("hash",
@@ -305,10 +321,12 @@ main (int argc,
                             " END "
                             "$$;"),
     GNUNET_PQ_make_execute ("CREATE TEMPORARY TABLE IF NOT EXISTS test_pq ("
-                            " tamount taler_amount NOT NULL"
+                            "tamount taler_amount NOT NULL"
                             ",json VARCHAR NOT NULL"
                             ",aamount taler_amount[]"
                             ",aamountc taler_amount_currency[]"
+                            ",aamountn taler_amount[] NOT NULL DEFAULT ARRAY[]::taler_amount[]"
+                            ",aamountnc taler_amount_currency[] NOT NULL DEFAULT ARRAY[]::taler_amount_currency[]"
                             ",tamountc taler_amount_currency"
                             ",hash gnunet_hashcode"
                             ",hashes gnunet_hashcode[]"
