@@ -18,12 +18,12 @@
  * @brief functions to initialize parameter arrays
  * @author Jonathan Buchanan
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include <sqlite3.h>
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_sq_lib.h>
-#include "taler_sq_lib.h"
-#include "taler_util.h"
+#include "taler/taler_sq_lib.h"
+#include "taler/taler_util.h"
 
 
 /**
