@@ -22,7 +22,7 @@
  * @brief testcase to test the mustach/jansson integration
  * @author Florian Dold
  */
-#include "platform.h"
+#include "taler/platform.h"
 #include "mustach-jansson.h"
 #include <gnunet/gnunet_util_lib.h>
 
