@@ -909,11 +909,14 @@ then
 
     echo -n "Setting up exchange keys ..."
     rm -f test_exchange_api_home/.local/share/taler-exchange/offline/secm_tofus.pub
+    NEXT_YEAR=$(expr 1 + $(date +%Y))
     taler-exchange-offline -c "$CONF" \
       download \
       sign \
       wire-fee now "$WIRE_DOMAIN" "$CURRENCY:0.01" "$CURRENCY:0.01" \
+      wire-fee "$NEXT_YEAR" "$WIRE_DOMAIN" "$CURRENCY:0.01" "$CURRENCY:0.01" \
       global-fee now "$CURRENCY:0.01" "$CURRENCY:0.01" "$CURRENCY:0.0" 1h 1year 5 \
+      global-fee "$NEXT_YEAR" "$CURRENCY:0.01" "$CURRENCY:0.01" "$CURRENCY:0.0" 1h 1year 5 \
       upload &> taler-exchange-offline.log
     echo "OK"
     ENABLED=$(taler-exchange-config -c "$CONF" -s "$USE_ACCOUNT" -o "ENABLE_CREDIT")
