@@ -21,18 +21,18 @@
  * @brief testcase to test exchange's age-restrictrition related HTTP API interfaces
  * @author Özgür Kesim
  */
-#include "platform.h"
-#include "taler_util.h"
-#include "taler_signatures.h"
-#include "taler_exchange_service.h"
-#include "taler_json_lib.h"
+#include "taler/platform.h"
+#include "taler/taler_util.h"
+#include "taler/taler_signatures.h"
+#include "taler/taler_exchange_service.h"
+#include "taler/taler_json_lib.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_testing_lib.h>
 #include <microhttpd.h>
-#include "taler_bank_service.h"
-#include "taler_fakebank_lib.h"
-#include "taler_testing_lib.h"
-#include "taler_extensions.h"
+#include "taler/taler_bank_service.h"
+#include "taler/taler_fakebank_lib.h"
+#include "taler/taler_testing_lib.h"
+#include "taler/taler_extensions.h"
 
 /**
  * Configuration file we use.  One (big) configuration is used
@@ -195,7 +195,6 @@ run (void *cls,
     TALER_TESTING_cmd_end ()
   };
 
-
   struct TALER_TESTING_Command refresh_age[] = {
     /* Fill reserve with EUR:5, 1ct is for fees. */
     CMD_TRANSFER_TO_EXCHANGE (
@@ -212,7 +211,7 @@ run (void *cls,
      */
     CMD_EXEC_WIREWATCH ("wirewatch-age-2"),
     /**
-     * Withdraw EUR:7 with age restriction for age 13.
+     * Withdraw EUR:5 with age restriction for age 13.
      */
     TALER_TESTING_cmd_withdraw_amount (
       "refresh-withdraw-coin-age-1",
