@@ -21,14 +21,14 @@
  * @brief command to check the /history/incoming API from the bank.
  * @author Marcello Stanisci
  */
-#include "platform.h"
-#include "taler_json_lib.h"
+#include "taler/platform.h"
+#include "taler/taler_json_lib.h"
 #include <gnunet/gnunet_curl_lib.h>
-#include "taler_exchange_service.h"
-#include "taler_testing_lib.h"
-#include "taler_fakebank_lib.h"
-#include "taler_bank_service.h"
-#include "taler_fakebank_lib.h"
+#include "taler/taler_exchange_service.h"
+#include "taler/taler_testing_lib.h"
+#include "taler/taler_fakebank_lib.h"
+#include "taler/taler_bank_service.h"
+#include "taler/taler_fakebank_lib.h"
 
 
 /**
