@@ -221,7 +221,7 @@ TALER_withdraw_master_seed_setup_random (
 
 void
 TALER_refresh_master_setup_random (
-  struct TALER_RefreshMasterSecretP *rms)
+  struct TALER_PublicRefreshMasterSeedP *rms)
 {
   GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_STRONG,
                               rms,
@@ -250,30 +250,6 @@ TALER_transfer_secret_to_planchet_secret (
 }
 
 
-void
-TALER_planchet_secret_to_transfer_priv (
-  const struct TALER_RefreshMasterSecretP *rms,
-  const struct TALER_CoinSpendPrivateKeyP *old_coin_priv,
-  uint32_t cnc_num,
-  struct TALER_TransferPrivateKeyP *tpriv)
-{
-  uint32_t be_salt = htonl (cnc_num);
-
-  GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CRYPTO_kdf (tpriv,
-                                    sizeof (*tpriv),
-                                    &be_salt,
-                                    sizeof (be_salt),
-                                    old_coin_priv,
-                                    sizeof (*old_coin_priv),
-                                    rms,
-                                    sizeof (*rms),
-                                    "taler-transfer-priv-derivation",
-                                    strlen ("taler-transfer-priv-derivation"),
-                                    NULL, 0));
-}
-
-
 void
 TALER_cs_withdraw_nonce_derive (
   const struct TALER_PlanchetMasterSecretP *ps,
@@ -309,8 +285,9 @@ TALER_cs_withdraw_seed_to_blinding_seed (
 
 
 void
-TALER_cs_refresh_secret_to_blinding_seed (
-  const struct TALER_RefreshMasterSecretP *secret,
+TALER_cs_refresh_seed_to_blinding_seed (
+  const struct TALER_PublicRefreshMasterSeedP *seed,
+  const struct TALER_CoinSpendPrivateKeyP *coin_priv,
   struct TALER_BlindingMasterSeedP *blinding_seed)
 {
   GNUNET_assert (GNUNET_YES ==
@@ -318,8 +295,10 @@ TALER_cs_refresh_secret_to_blinding_seed (
                                     sizeof (*blinding_seed),
                                     "refresh-blinding",
                                     strlen ("refresh-blinding"),
-                                    secret,
-                                    sizeof(*secret),
+                                    coin_priv,
+                                    sizeof(*coin_priv),
+                                    seed,
+                                    sizeof(*seed),
                                     NULL,
                                     0));
 }
@@ -387,28 +366,6 @@ TALER_cs_derive_only_cs_blind_nonces_from_seed (
 }
 
 
-void
-TALER_cs_refresh_nonce_derive (
-  const struct TALER_RefreshMasterSecretP *rms,
-  uint32_t coin_num_salt,
-  struct GNUNET_CRYPTO_CsSessionNonce *nonce)
-{
-  uint32_t be_salt = htonl (coin_num_salt);
-
-  GNUNET_assert (GNUNET_YES ==
-                 GNUNET_CRYPTO_kdf (nonce,
-                                    sizeof (*nonce),
-                                    &be_salt,
-                                    sizeof (be_salt),
-                                    "refresh-n",
-                                    strlen ("refresh-n"),
-                                    rms,
-                                    sizeof(*rms),
-                                    NULL,
-                                    0));
-}
-
-
 bool
 TALER_cs_mark_indices (
   size_t num,
@@ -568,83 +525,11 @@ TALER_planchet_to_coin (
 
 
 void
-TALER_refresh_get_commitment (struct TALER_RefreshCommitmentP *rc,
-                              uint32_t kappa,
-                              const struct TALER_RefreshMasterSecretP *rms,
-                              uint32_t num_new_coins,
-                              const struct TALER_RefreshCommitmentEntry *rcs,
-                              const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                              const struct TALER_Amount *amount_with_fee)
-{
-  struct GNUNET_HashContext *hash_context;
-
-  hash_context = GNUNET_CRYPTO_hash_context_start ();
-  if (NULL != rms)
-    GNUNET_CRYPTO_hash_context_read (hash_context,
-                                     rms,
-                                     sizeof (*rms));
-  /* first, iterate over transfer public keys for hash_context */
-  for (unsigned int i = 0; i<kappa; i++)
-  {
-    GNUNET_CRYPTO_hash_context_read (hash_context,
-                                     &rcs[i].transfer_pub,
-                                     sizeof (struct TALER_TransferPublicKeyP));
-  }
-  /* next, add all of the hashes from the denomination keys to the
-     hash_context */
-  for (unsigned int i = 0; i<num_new_coins; i++)
-  {
-    struct TALER_DenominationHashP denom_hash;
-
-    /* The denomination keys should / must all be identical regardless
-       of what offset we use, so we use [0]. */
-    GNUNET_assert (kappa > 0); /* sanity check */
-    TALER_denom_pub_hash (rcs[0].new_coins[i].dk,
-                          &denom_hash);
-    GNUNET_CRYPTO_hash_context_read (hash_context,
-                                     &denom_hash,
-                                     sizeof (denom_hash));
-  }
-
-  /* next, add public key of coin and amount being refreshed */
-  {
-    struct TALER_AmountNBO melt_amountn;
-
-    GNUNET_CRYPTO_hash_context_read (hash_context,
-                                     coin_pub,
-                                     sizeof (struct TALER_CoinSpendPublicKeyP));
-    TALER_amount_hton (&melt_amountn,
-                       amount_with_fee);
-    GNUNET_CRYPTO_hash_context_read (hash_context,
-                                     &melt_amountn,
-                                     sizeof (struct TALER_AmountNBO));
-  }
-
-  /* finally, add all the envelopes */
-  for (unsigned int i = 0; i<kappa; i++)
-  {
-    const struct TALER_RefreshCommitmentEntry *rce = &rcs[i];
-
-    for (unsigned int j = 0; j<num_new_coins; j++)
-    {
-      const struct TALER_RefreshCoinData *rcd = &rce->new_coins[j];
-
-      TALER_blinded_planchet_hash_ (&rcd->blinded_planchet,
-                                    hash_context);
-    }
-  }
-
-  /* Conclude */
-  GNUNET_CRYPTO_hash_context_finish (hash_context,
-                                     &rc->session_hash);
-}
-
-
-void
-TALER_refresh_get_commitment_v27 (
+TALER_refresh_get_commitment (
   struct TALER_RefreshCommitmentP *rc,
   const struct TALER_PublicRefreshMasterSeedP *refresh_seed,
   const struct TALER_BlindingMasterSeedP *blinding_seed,
+  const struct TALER_KappaTransferPublicKeys *k_tpbs,
   const struct TALER_KappaHashBlindedPlanchetsP *k_bps_h,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
   const struct TALER_Amount *amount_with_fee)
@@ -701,23 +586,133 @@ TALER_refresh_get_commitment_v27 (
 
 
 void
-TALER_refresh_master_secret_to_refresh_seed (
-  const struct TALER_RefreshMasterSecretP *rms,
-  struct TALER_PublicRefreshMasterSeedP *r_seed)
+TALER_refresh_expand_seed_to_kappa_batch_seeds (
+  const struct TALER_PublicRefreshMasterSeedP *refresh_master_seed,
+  const struct TALER_CoinSpendPrivateKeyP *coin_priv,
+  struct TALER_KappaPrivateRefreshBatchSeedsP *kappa_batch_seeds)
 {
   GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CRYPTO_kdf (r_seed,
-                                    sizeof (*r_seed),
-                                    "refresh-seed",
-                                    strlen ("refresh-seed"),
-                                    rms,
-                                    sizeof (*rms),
+                 GNUNET_CRYPTO_kdf (kappa_batch_seeds,
+                                    sizeof (*kappa_batch_seeds),
+                                    "refresh-batch-seeds",
+                                    strlen ("refresh-batch-seeds"),
+                                    refresh_master_seed,
+                                    sizeof (*refresh_master_seed),
+                                    coin_priv,
+                                    sizeof(*coin_priv),
                                     NULL, 0));
 }
 
 
 void
-TALER_refresh_expand_kappa_nonces (
+TALER_refresh_expand_batch_seed_to_transfer_private_keys (
+  const struct TALER_PrivateRefreshBatchSeedP *batch_seed,
+  size_t num_transfer_pks,
+  struct TALER_TransferPrivateKeyP transfer_pks[num_transfer_pks])
+{
+  GNUNET_assert (GNUNET_OK ==
+                 GNUNET_CRYPTO_kdf (
+                   transfer_pks,
+                   sizeof (*transfer_pks) * num_transfer_pks,
+                   "refresh-transfer-private-keys",
+                   strlen ("refresh-transfer-private-keys"),
+                   batch_seed,
+                   sizeof (*batch_seed),
+                   NULL, 0));
+}
+
+
+void
+TALER_refresh_expand_batch_seed_to_transfer_secrets (
+  const struct TALER_PrivateRefreshBatchSeedP *batch_seed,
+  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  size_t num_transfer_secrets,
+  struct TALER_TransferSecretP transfer_secrets[num_transfer_secrets])
+{
+  struct TALER_TransferPrivateKeyP transfer_pks[num_transfer_secrets];
+
+  TALER_refresh_expand_batch_seed_to_transfer_private_keys (
+    batch_seed,
+    num_transfer_secrets,
+    transfer_pks);
+
+  for (size_t i = 0; i < num_transfer_secrets; i++)
+  {
+    TALER_link_reveal_transfer_secret (
+      &transfer_pks[i],
+      coin_pub,
+      &transfer_secrets[i]);
+  }
+}
+
+
+void
+TALER_refresh_expand_batch_seed_to_planchet_master_secrets (
+  const struct TALER_PrivateRefreshBatchSeedP *batch_seed,
+  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  size_t num_planchet_secrets,
+  struct TALER_PlanchetMasterSecretP planchet_secrets[num_planchet_secrets])
+{
+  struct TALER_TransferPrivateKeyP transfer_pks[num_planchet_secrets];
+  struct TALER_TransferSecretP transfer_secrets[num_planchet_secrets];
+
+  TALER_refresh_expand_batch_seed_to_transfer_private_keys (
+    batch_seed,
+    num_planchet_secrets,
+    transfer_pks);
+
+  for (size_t i = 0; i < num_planchet_secrets; i++)
+  {
+    TALER_link_reveal_transfer_secret (
+      &transfer_pks[i],
+      coin_pub,
+      &transfer_secrets[i]);
+
+    TALER_transfer_secret_to_planchet_secret (
+      &transfer_secrets[i],
+      i,
+      &planchet_secrets[i]);
+  }
+}
+
+
+void
+TALER_refresh_expand_batch_seed_to_transfer_data (
+  const struct TALER_PrivateRefreshBatchSeedP *batch_seed,
+  const struct TALER_CoinSpendPublicKeyP *coin_pub,
+  size_t num,
+  struct TALER_PlanchetMasterSecretP planchet_secrets[num],
+  struct TALER_TransferPublicKeyP transfer_pubs[num])
+{
+  struct TALER_TransferPrivateKeyP transfer_pks[num];
+  struct TALER_TransferSecretP transfer_secrets[num];
+
+  TALER_refresh_expand_batch_seed_to_transfer_private_keys (
+    batch_seed,
+    num,
+    transfer_pks);
+
+  for (size_t i = 0; i < num; i++)
+  {
+    TALER_link_reveal_transfer_secret (
+      &transfer_pks[i],
+      coin_pub,
+      &transfer_secrets[i]);
+
+    TALER_transfer_secret_to_planchet_secret (
+      &transfer_secrets[i],
+      i,
+      &planchet_secrets[i]);
+
+    GNUNET_CRYPTO_ecdhe_key_get_public (
+      &transfer_pks[i].ecdhe_priv,
+      &transfer_pubs[i].ecdhe_pub);
+  }
+}
+
+
+void
+TALER_refresh_expand_kappa_nonces_v27 (
   const struct TALER_PublicRefreshMasterSeedP *refresh_seed,
   struct TALER_KappaPublicRefreshNoncesP *kappa_nonces)
 {
@@ -733,7 +728,7 @@ TALER_refresh_expand_kappa_nonces (
 
 
 void
-TALER_refresh_signature_to_secrets (
+TALER_refresh_signature_to_secrets_v27 (
   const struct TALER_PrivateRefreshNonceSignatureP *sig,
   size_t num_secrets,
   struct TALER_PlanchetMasterSecretP secrets[static num_secrets])
