@@ -343,7 +343,7 @@ more:
 enum TALER_ErrorCode
 TALER_CRYPTO_helper_esign_sign_ (
   struct TALER_CRYPTO_ExchangeSignHelper *esh,
-  const struct GNUNET_CRYPTO_EccSignaturePurpose *purpose,
+  const struct GNUNET_CRYPTO_SignaturePurpose *purpose,
   struct TALER_ExchangePublicKeyP *exchange_pub,
   struct TALER_ExchangeSignatureP *exchange_sig)
 {
@@ -360,7 +360,7 @@ TALER_CRYPTO_helper_esign_sign_ (
                  UINT16_MAX - sizeof (struct TALER_CRYPTO_EddsaSignRequest));
   {
     char buf[sizeof (struct TALER_CRYPTO_EddsaSignRequest) + purpose_size
-             - sizeof (struct GNUNET_CRYPTO_EccSignaturePurpose)];
+             - sizeof (struct GNUNET_CRYPTO_SignaturePurpose)];
     struct TALER_CRYPTO_EddsaSignRequest *sr
       = (struct TALER_CRYPTO_EddsaSignRequest *) buf;
 
