@@ -931,6 +931,7 @@ int
 main (int argc,
       const char *const argv[])
 {
+  const char *loglevel = "WARNING";
   struct GNUNET_OS_Process *helper;
   char *libexec_dir;
   char *binary_name;
@@ -943,7 +944,7 @@ main (int argc,
   unsetenv ("XDG_DATA_HOME");
   unsetenv ("XDG_CONFIG_HOME");
   GNUNET_log_setup ("test-helper-rsa",
-                    "WARNING",
+                    loglevel,
                     NULL);
   libexec_dir = GNUNET_OS_installation_get_path (TALER_EXCHANGE_project_data (),
                                                  GNUNET_OS_IPK_BINDIR);
@@ -959,7 +960,7 @@ main (int argc,
                                     "-c",
                                     "test_helper_rsa.conf",
                                     "-L",
-                                    "WARNING",
+                                    loglevel,
                                     NULL);
   if (NULL == helper)
   {
